within DCOL.Consumption.Fluid.HeatExchangers.InProgress;
model fullDryCoil
  "Cooling coil (does not dehumidify), water in path a2-b2, air in path a1-b1"
  //Reference Literature: BRIAN KEITH KASTL, DEHUMIDIFYING COIL MODELS FOR ENERGY SIMULATION AND DESIGN, Master Thesis
  extends BaseClasses.PartialEffectiveness(
    redeclare replaceable package Medium1 =
        Modelica.Media.Interfaces.PartialCondensingGases,
    sensibleOnly1=true,
    sensibleOnly2=true,
    Q1_flow=-Q_flow,
    Q2_flow=Q_flow,
    mWat1_flow=mWat_flow,
    mWat2_flow=0,
    final prescribedHeatFlowRate1=true,
    final prescribedHeatFlowRate2=true);

  Modelica.Blocks.Interfaces.RealInput fractionUse "fraction of coil Used"
    annotation (Placement(transformation(extent={{-140,80},{-100,120}})));

  parameter Types.HXConfiguration config "heat exchanger configuration";
  //parameter Modelica.SIunits.Mass m_c "mass capacity of coil";
  parameter Modelica.SIunits.Temp_C Ti0_a
    "temperature input nominal, port a1 (air)"
    annotation (Dialog(tab="General", group="Nominal condition"));
  parameter Real RHi0 "relative humidity input nominal, port a1 (air)"
    annotation (Dialog(tab="General", group="Nominal condition"));
  parameter Modelica.SIunits.Temp_C To0_a
    "temperature output nominal, port b1 (air)"
    annotation (Dialog(tab="General", group="Nominal condition"));
  parameter Real RHo0 "relative humidity output nominal, port b1 (air)"
    annotation (Dialog(tab="General", group="Nominal condition"));
  parameter Modelica.SIunits.Temp_C Ti0_w
    "temperature input nominal, port a2 (water)"
    annotation (Dialog(tab="General", group="Nominal condition"));
  parameter Modelica.SIunits.Temp_C To0_w
    "temperature output nominal, port b2 (water)"
    annotation (Dialog(tab="General", group="Nominal condition"));
  parameter Real m=0.6
    "Constant defining system characteristics, normally set to 0.6";

protected
  final parameter Modelica.SIunits.MassFraction Wi0=
      Psychrometrics.Functions.RHtoW(
      p=Medium1.p_default,
      RH=RHi0,
      T=Ti0_a);
  final parameter Modelica.SIunits.MassFraction Wo0=
      Psychrometrics.Functions.RHtoW(
      p=Medium1.p_default,
      RH=RHo0,
      T=To0_a);
  final parameter Medium1.ThermodynamicState sta1_default=Medium1.setState_pTX(
      T=Ti0_a,
      p=Medium1.p_default,
      X={Wi0/(1 + Wi0),1 - Wi0/(1 + Wi0)})
    "Default state for medium 1 (only for wet air)";
  //
  final parameter Medium2.ThermodynamicState sta2_default=Medium2.setState_pTX(
      T=To0_a,
      p=Medium2.p_default,
      X=Medium2.X_default[1:Medium2.nXi]) "Default state for medium 2";
  parameter Integer i1_w(min=1, fixed=false) "Index for water substance";

  parameter Real r(start=0.75, fixed=true);
  //ratio of convection heat transfer
  parameter Modelica.SIunits.ThermalResistance R0(start=1e-4, fixed=false);
  //overall coefficient of heat transfer nominal
  parameter Modelica.SIunits.MassFlowRate mflow0_a=m1_flow_nominal
    "mass flow nominal, air";
  parameter Modelica.SIunits.MassFlowRate mflow0_w=m2_flow_nominal
    "mass flow nominal, water";

  Real b;

  Modelica.SIunits.HeatFlowRate Q_flow;
  Modelica.SIunits.MassFlowRate mWat_flow;

  Modelica.SIunits.MassFlowRate mflow_a(start=mflow0_a);
  //mass flow, air
  Modelica.SIunits.Temp_C Ti_db(start=Ti0_a);
  //temperature input, dry bulb air
  Modelica.SIunits.MassFlowRate mflow_w(start=mflow0_w);
  //mass flow, water
  Modelica.SIunits.Temp_C Ti_w(start=Ti0_w);
  //temperature input, water
  Modelica.SIunits.ThermalConductance Cflow_w;
  //heat flow capacity, water
  Modelica.SIunits.ThermalConductance Cflow_a;
  //heat flow capacity, air
  Real eff(start=0.8);
  //effectiveness, dry regime
  Real NTU(start=1.0);
  //number of exchanger heat transfer units in the dry regime
  Real Z;
  //dimensionless capacity rate ratio, dry regime
  Modelica.SIunits.ThermalResistance R(start=1.0);
  //thermal resistance

initial equation

  (Ti0_a - To0_a)*Medium1.specificHeatCapacityCp(sta1_default)*mflow0_a = min(mflow0_a*Medium1.specificHeatCapacityCp(sta1_default), mflow0_w*Medium2.specificHeatCapacityCp(sta2_default))*(Ti0_a - Ti0_w)*Psychrometrics.Functions.eff(
    config,
    min(mflow0_a*Medium1.specificHeatCapacityCp(sta1_default), mflow0_w*Medium2.specificHeatCapacityCp(sta2_default))/max(mflow0_a*Medium1.specificHeatCapacityCp(sta1_default), mflow0_w*Medium2.specificHeatCapacityCp(sta2_default)),
    1/(R0*min(mflow0_a*Medium1.specificHeatCapacityCp(sta1_default), mflow0_w*Medium2.specificHeatCapacityCp(sta2_default))));

initial algorithm
  i1_w := -1;
  for i in 1:Medium1.nXi loop
    if Modelica.Utilities.Strings.isEqual(
        string1=Medium1.substanceNames[i],
        string2="Water",
        caseSensitive=false) then
      i1_w := i;
    end if;
  end for;
  assert(i1_w > 0, "Substance 'water' is not present in Medium1 '" + Medium1.mediumName
     + "'.\n" + "Check medium model.");
equation
  //########################################################## CONNECTORS ############################################################
  //=========================================================== Input ==============================================================
  //*********************************************************** Air ***************************************************************
  Ti_db = Modelica.SIunits.Conversions.to_degC(T_in1);

  mflow_a = max(abs(m1_flow), 1e-4*abs(mflow0_a));
  //********************************************************** Water **************************************************************
  Ti_w = Modelica.SIunits.Conversions.to_degC(T_in2);

  mflow_w = max(abs(m2_flow), 1e-4*abs(mflow0_w));
  //########################################################## energy balance ############################################################
  Q_flow = eff*min(Cflow_a, Cflow_w)*(Ti_db - Ti_w);
  //*******************************************************************
  eff = Psychrometrics.Functions.eff(
    config,
    Z,
    NTU);

  NTU = fractionUse/(R*min(Cflow_a, Cflow_w));

  Z = min(Cflow_a, Cflow_w)/max(Cflow_a, Cflow_w);

  Cflow_a = max(C1_flow, 1e-4);

  Cflow_w = max(C2_flow, 1e-4);

  //########################################################## RESISTANCE ############################################################
  b = ((Ti0_w - Ti0_a)/(Ti_w - Ti_db + (Ti0_w - Ti0_a))) + (((mflow0_w/(mflow_w
     + mflow0_w)) + (mflow0_a/(mflow_a + mflow0_a)))/2);

  R/R0 = (1 + b) - b*((1/(1 + r))*(mflow_a/mflow0_a) + (r/(1 + r))*(mflow_w/
    mflow0_w));

  mWat_flow = 0;

end fullDryCoil;
