within DCOL.Consumption.Fluid.HeatExchangers.InProgress;
model fullWetCoil_extend_partialEff
  "Cooling and Dehumidifying coil (always dehumidifies), water in path a2-b2, air in path a1-b1"
  //Reference Literature: BRIAN KEITH KASTL, DEHUMIDIFYING COIL MODELS FOR ENERGY SIMULATION AND DESIGN, Master Thesis
  extends BaseClasses.PartialEffectiveness(
    redeclare replaceable package Medium1 =
        Modelica.Media.Interfaces.PartialCondensingGases,
    sensibleOnly1=false,
    sensibleOnly2=true,
    Q1_flow=-Q_flow,
    Q2_flow=Q_flow,
    mWat1_flow=mWat_flow,
    mWat2_flow=0,
    final prescribedHeatFlowRate1=true,
    final prescribedHeatFlowRate2=true);

  Modelica.Blocks.Interfaces.RealInput fractionUse "fraction of coil Used"
    annotation (Placement(transformation(extent={{-140,80},{-100,120}})));

  parameter Types.HXConfiguration config "heat exchanger configuration";
  //parameter Modelica.SIunits.Mass m_c "mass capacity of coil";
  parameter Modelica.SIunits.Temp_C Ti0_a
    "temperature input nominal, port a1 (air)"
    annotation (Dialog(tab="General", group="Nominal condition"));
  parameter Real RHi0 "relative humidity input nominal, port a1 (air)"
    annotation (Dialog(tab="General", group="Nominal condition"));
  parameter Modelica.SIunits.Temp_C To0_a
    "temperature output nominal, port b1 (air)"
    annotation (Dialog(tab="General", group="Nominal condition"));
  parameter Real RHo0 "relative humidity output nominal, port b1 (air)"
    annotation (Dialog(tab="General", group="Nominal condition"));
  parameter Modelica.SIunits.Temp_C Ti0_w
    "temperature input nominal, port a2 (water)"
    annotation (Dialog(tab="General", group="Nominal condition"));
  parameter Modelica.SIunits.Temp_C To0_w
    "temperature output nominal, port b2 (water)"
    annotation (Dialog(tab="General", group="Nominal condition"));
  parameter Real m=0.6
    "Constant defining system characteristics, normally set to 0.6";

protected
  final parameter Modelica.SIunits.MassFraction Wi0=
      Psychrometrics.Functions.RHtoW(
      p=Medium1.p_default,
      RH=RHi0,
      T=Ti0_a);
  final parameter Modelica.SIunits.MassFraction Wo0=
      Psychrometrics.Functions.RHtoW(
      p=Medium1.p_default,
      RH=RHo0,
      T=To0_a);

  final parameter Modelica.SIunits.Temp_C Ti0_wb=
      Psychrometrics.Functions.Twb(
      Tdb=Ti0_a,
      W=Wi0,
      WAs=Psychrometrics.Functions.Ws(                 p=p_a, T=Ti0_wb));
  //wet buld temperature input nominal, air
  final parameter Modelica.SIunits.Temp_C To0_wb=
      Psychrometrics.Functions.Twb(
      Tdb=To0_a,
      W=Wo0,
      WAs=Psychrometrics.Functions.Ws(                 p=p_a, T=To0_wb));
  //wet buld temperature output nominal, air

  final parameter Modelica.SIunits.SpecificEnthalpy hi_a0=
      Psychrometrics.Functions.h(                 Tdb=Ti0_a, W=Wi0);
  //enthalpy input nomina, air
  final parameter Modelica.SIunits.SpecificEnthalpy ho_a0=
      Psychrometrics.Functions.h(                 Tdb=To0_a, W=Wo0);

  final parameter Modelica.SIunits.SpecificEnthalpy hwsat_in0=
      Psychrometrics.Functions.h(                 Tdb=Ti0_w, W=
      Psychrometrics.Functions.Ws(                 T=Ti0_w, p=p_a));
  //saturation enthalpy at water input conditions
  final parameter Modelica.SIunits.SpecificEnthalpy hwsat_out0=
      Psychrometrics.Functions.h(                 Tdb=To0_w, W=
      Psychrometrics.Functions.Ws(                 T=To0_w, p=p_a));
  //saturation enthalpy at water output conditions

  final parameter Modelica.SIunits.SpecificEnthalpy delta_h=((hi_a0 -
      hwsat_out0) - (ho_a0 - hwsat_in0))/log((hi_a0 - hwsat_out0) - (ho_a0 -
      hwsat_in0));

  final parameter Medium1.ThermodynamicState sta1_default=Medium1.setState_pTX(
      T=Ti0_a,
      p=Medium1.p_default,
      X={Wi0/(1 + Wi0),1 - Wi0/(1 + Wi0)})
    "Default state for medium 1 (only for wet air)";
  //
  final parameter Medium2.ThermodynamicState sta2_default=Medium2.setState_pTX(
      T=To0_a,
      p=Medium2.p_default,
      X=Medium2.X_default[1:Medium2.nXi]) "Default state for medium 2";
  parameter Integer i1_w(min=1, fixed=false) "Index for water substance";

  Medium1.MassFraction X_w_in1 "Inlet water mass fraction of medium 1";

  parameter Real r(start=0.75, fixed=true);
  //ratio of convection heat transfer
  parameter Modelica.SIunits.ThermalResistance Rerror(start=1e-8);
  //error due to calculate R and R_wet
  parameter Modelica.SIunits.ThermalResistance R0=(4.3/3.3)*delta_h/(mflow0_a*(
      hi_a0 - ho_a0)*Medium1.specificHeatCapacityCp(sta1_default));
  //overall coefficient of heat transfer nominal,   //assuming UA=1/R and UAh=UA/cp --> see Chapter 16 EnergyPlus Engineering Reference --> also see "An Improved Simple Chilled Water Cooling Coil Model" from Wang 2012
  parameter Modelica.SIunits.ThermalResistance R0_a=R0*r/(1 + r);
  //overall coefficient of heat transfer nominal, air
  parameter Modelica.SIunits.MassFlowRate mflow0_a=m1_flow_nominal
    "mass flow nominal, air";
  parameter Modelica.SIunits.MassFlowRate mflow0_w=m2_flow_nominal
    "mass flow nominal, water";

  Real b;

  Modelica.SIunits.HeatFlowRate Q_flow;
  //Total heat flow rate
  Modelica.SIunits.HeatFlowRate Q_flow_s;
  //Sensible heat flow rate
  Modelica.SIunits.HeatFlowRate Q_flow_l;
  //Latent heat flow rate
  Modelica.SIunits.MassFlowRate mWat_flow;

  Modelica.SIunits.MassFlowRate mflow_a(start=mflow0_a);
  //mass flow, air
  Modelica.SIunits.Temp_C Ti_db(start=Ti0_a);
  //temperature input, dry bulb air
  //Modelica.SIunits.Temp_C To_db;
  //temperature input, dry bulb air
  Modelica.SIunits.Temp_C Ti_wb(start=Ti0_a - 5);
  //temperature input, wet bulb air
  //   Modelica.SIunits.MassFraction Wo;
  //   //humidity ratio, output

  Modelica.SIunits.MassFlowRate mflow_w(start=mflow0_w);
  //mass flow, water
  Modelica.SIunits.Temp_C Ti_w(start=Ti0_w);
  //temperature input, water
  Modelica.SIunits.ThermalConductance Cflow_w;
  //heat flow capacity, water
  Modelica.SIunits.ThermalConductance Cflow_a;
  //heat flow capacity, air
  Modelica.SIunits.ThermalConductance Cflow_a_wet(fixed=false);
  //heat flow capacity, air in the wet regime
  Modelica.SIunits.Temp_C To_db_wet(start=To0_a);
  //temperature output, dry bulb air in the wet regime
  Modelica.SIunits.Temp_C To_wb_wet(start=To0_a - 5);
  //temperature output, wet bulb air in the wet regime
  //Modelica.SIunits.Temp_C To_w_wet;
  //temperature output, water in the wet regime
  Real eff;
  //effectiveness, wet regime
  Real NTU(start=1.0);
  //number of exchanger heat transfer units in the wet regime
  Real Z;
  //dimensionless capacity rate ratio, wet regime
  Modelica.SIunits.ThermalResistance R;
  //thermal resistance
  Modelica.SIunits.ThermalResistance R_a;
  //thermal resistance, air
  Modelica.SIunits.ThermalResistance R_wet;
  //thermal resistance, wet regime
  Modelica.SIunits.ThermalResistance R_a_wet;
  //thermal resistance, air in the wet regime
  //final parameter Modelica.SIunits.SpecificHeatCapacity cf_a = (hi_a0 - ho_a0)/(Ti0_wb - To0_wb);
  final parameter Modelica.SIunits.SpecificHeatCapacity cf_a=(hwsat_out0 -
      hwsat_in0)/(To0_w - Ti0_w);
  //saturated specific heat capacity, air;
  //========================================================= Contact ==============================================================
  Modelica.SIunits.Temp_C Tc_wet(start=To0_w - 2);
  //temperature, contact in the wet regime
  //   Real effc_wet(start=0.5);
  //effectiveness, contact in the wet regime
  //   Real NTUc_wet(start=1.0);
  //number of exchanger heat transfer units, contact
  //Modelica.SIunits.Temp_C Tc_wet_wb(start=Ti0_w + 2);

  //######################################################### PSYCHROMETRICS #########################################################
  Modelica.SIunits.SpecificHeatCapacity c_a "specific heat capacity, air";
  //Modelica.SIunits.SpecificHeatCapacity c_w "specific heat capacity, water";
  //=========================================================== Input ==================ss============================================
  Modelica.SIunits.SpecificEnthalpy hi_a;
  //enthalpy input, air;
  Modelica.SIunits.MassFraction Wi;
  //humidity ratio, input
  //=========================================================== Output =============================================================
  Modelica.SIunits.SpecificEnthalpy ho_a;
  //enthalpy output, air;
  Modelica.SIunits.MassFraction Wo_wet(start=Wo0);
  //humidity ratio in the wet regime, output
  Modelica.SIunits.MassFraction Xo_wet;
  //Species content output of Medium 1
  //=========================================================== Contact =============================================================
  Modelica.SIunits.MassFraction Wc_wet;
  Modelica.SIunits.SpecificEnthalpy hc_wet;

  Real RHo(start=0.5);

  parameter Modelica.SIunits.Pressure p_a=Medium1.pStp;

initial algorithm
  i1_w := -1;
  for i in 1:Medium1.nXi loop
    if Modelica.Utilities.Strings.isEqual(
        string1=Medium1.substanceNames[i],
        string2="Water",
        caseSensitive=false) then
      i1_w := i;
    end if;
  end for;
  assert(i1_w > 0, "Substance 'water' is not present in Medium1 '" + Medium1.mediumName
     + "'.\n" + "Check medium model.");
equation
  //########################################################## CONNECTORS ############################################################
  //=========================================================== Input ==============================================================
  //*********************************************************** Air ***************************************************************
  Ti_db = Modelica.SIunits.Conversions.to_degC(T_in1);

  X_w_in1 = Modelica.Fluid.Utilities.regStep(
    mflow_a,
    state_a1_inflow.X[i1_w],
    state_b1_inflow.X[i1_w],
    m1_flow_small);
  Wi = X_w_in1/(1 - X_w_in1);

  mflow_a = max(abs(m1_flow), 1e-4*abs(mflow0_a));

  Cflow_a = max(C1_flow, 1e-4);

  //********************************************************** Water **************************************************************
  Ti_w = Modelica.SIunits.Conversions.to_degC(T_in2);

  mflow_w = max(abs(m2_flow), 1e-4*abs(mflow0_w));

  Cflow_w = max(C2_flow, 1e-4);

  //########################################################## WET REGIME ############################################################
  //=========================================================== Coil ===============================================================
  c_a = Medium1.specificHeatCapacityCp(state_a1_inflow);

  //c_w = Medium2.specificHeatCapacityCp(state_a2_inflow);

  Q_flow = mflow_a*(hi_a - ho_a);

  Q_flow_s = Cflow_a*(Ti_db - To_db_wet);

  Q_flow = Q_flow_s + Q_flow_l;

  eff = Psychrometrics.Functions.eff(
    config,
    Z,
    NTU);

  NTU = fractionUse/(R_wet*min(Cflow_a_wet, Cflow_w));
  //here is where I am inputing fraciton of use for partially wet/ partially dry coil

  Z = min(Cflow_a_wet, Cflow_w)/max(Cflow_a_wet, Cflow_w);

  Cflow_a_wet = max(Cflow_a*cf_a/c_a, 1e-4);

  //########################################################## RESISTANCE ############################################################
  b = ((Ti0_w - Ti0_a)/(Ti_w - Ti_db + (Ti0_w - Ti0_a))) + (((mflow0_w/(mflow_w
     + mflow0_w)) + (mflow0_a/(mflow_a + mflow0_a)))/2);

  R/R0 = (1 + b) - b*((1/(1 + r))*(mflow_a/mflow0_a) + (r/(1 + r))*(mflow_w/
    mflow0_w));

  R_a/R0_a = (mflow0_a/mflow_a)^m;

  R_a_wet = R_a*c_a/cf_a;

  R_wet - R_a_wet + Rerror = R - R_a;

  //######################################################## PSYCHROMETRICS ##########################################################
  //=========================================================== Input ==============================================================
  hi_a = Psychrometrics.Functions.h(Tdb=Ti_db, W=Wi);

  Wi = Psychrometrics.Functions.W2(
    Tdb=Ti_db,
    Twb=Ti_wb,
    WAs=Psychrometrics.Functions.Ws(p=p_a, T=Ti_wb));

  //=========================================================== Output =============================================================
  ho_a = Psychrometrics.Functions.h(Tdb=To_db_wet, W=Wo_wet);

  ho_a = hi_a - eff*(hi_a - hc_wet);

  Wo_wet = Wi - eff*(Wi - Wc_wet);

  Wo_wet = Psychrometrics.Functions.W2(
    Tdb=To_db_wet,
    Twb=To_wb_wet,
    WAs=Psychrometrics.Functions.Ws(p=p_a, T=To_wb_wet));

  Xo_wet = Wo_wet/(1 + Wo_wet);

  Wo_wet = Psychrometrics.Functions.RHtoW(
    p=p_a,
    RH=RHo,
    T=To_db_wet);

  //checked the base models, it requires total Qflow, odd?
  Q_flow_l = -2465.56*c_a*mWat_flow;
  //in theory this should avoid RH>1.0

  //========================================================== Contact =============================================================
  Tc_wet = 0.5*(Ti_w + (Ti_w + mflow_a*(hi_a - ho_a)/Cflow_w));
  //"average" contact surface temperature

  hc_wet = Psychrometrics.Functions.h(Tdb=Tc_wet, W=Wc_wet);

  Wc_wet = Psychrometrics.Functions.RHtoW(
    p=p_a,
    RH=1.0,
    T=Tc_wet);

end fullWetCoil_extend_partialEff;
