within DCOL.Consumption.Fluid.MixingVolumes;
model HydraulicSeparator "Model of an Hydraulic Separator"
  extends Interfaces.PartialFourPort(
    redeclare package Medium2 = Medium1,
    final allowFlowReversal1=allowFlowReversal,
    final allowFlowReversal2=allowFlowReversal);

  parameter Boolean allowFlowReversal=true;
  parameter Modelica.SIunits.MassFlowRate m_flow_nominal
    "Nominal mass flow rate of either primary or secundary used for regularisation near zero flow";
  parameter Modelica.SIunits.Diameter D "Diameter of the tank";
  parameter Modelica.SIunits.Diameter Dflange "Diameter of the flanges";
  parameter Modelica.SIunits.Length L1
    "Height between the top and the lowest of the supply flanges";
  parameter Modelica.SIunits.Length L2
    "Height between the lowest of the supply flanges and the highest of the return flanges";
  parameter Modelica.SIunits.Length L3
    "Heght between the highest of the return flanges and the bottom";

protected
  MixingVolume vol(
    redeclare package Medium = Medium1,
    nPorts=3,
    allowFlowReversal=allowFlowReversal1,
    V=(Modelica.Constants.pi/4)*(D^2)*L1,
    m_flow_nominal=m_flow_nominal) annotation (Placement(transformation(extent={{-10,60},{10,80}})));

  MixingVolume vol1(
    redeclare package Medium = Medium1,
    nPorts=3,
    allowFlowReversal=allowFlowReversal2,
    V=(Modelica.Constants.pi/4)*(D^2)*L3,
    m_flow_nominal=m_flow_nominal) annotation (Placement(transformation(extent={{-10,-60},{10,-80}})));
public
  Modelica.Fluid.Pipes.StaticPipe primarySupply(
    redeclare package Medium = Medium1,
    length=0.05,
    diameter=Dflange,
    allowFlowReversal=allowFlowReversal1)
    annotation (Placement(transformation(extent={{-60,70},{-40,50}})));
  Modelica.Fluid.Pipes.StaticPipe primaryReturn(
    redeclare package Medium = Medium1,
    length=0.05,
    diameter=Dflange,
    allowFlowReversal=allowFlowReversal2)
    annotation (Placement(transformation(extent={{-40,-70},{-60,-50}})));
  Modelica.Fluid.Pipes.DynamicPipe secondarySupply(
    redeclare package Medium = Medium1,
    length=0.05,
    diameter=Dflange,
    allowFlowReversal=allowFlowReversal1,
    use_HeatTransfer=true)
    annotation (Placement(transformation(extent={{40,70},{60,50}})));
  Modelica.Fluid.Pipes.DynamicPipe secondaryReturn(
    redeclare package Medium = Medium1,
    length=0.05,
    diameter=Dflange,
    allowFlowReversal=allowFlowReversal2,
    use_HeatTransfer=true)
    annotation (Placement(transformation(extent={{60,-70},{40,-50}})));
  Modelica.Fluid.Pipes.StaticPipe tankBody(
    redeclare package Medium = Medium1,
    allowFlowReversal=allowFlowReversal2,
    diameter=D,
    length=L2,
    height_ab=L2) annotation (Placement(transformation(
        extent={{10,-10},{-10,10}},
        rotation=270,
        origin={0,-30})));
  Modelica.Fluid.Sensors.MassFlowRate senMasFlo(redeclare package Medium =
        Medium1, allowFlowReversal=allowFlowReversal)
    "Mass exhange between 1 and 2" annotation (Placement(transformation(
        extent={{-10,-10},{10,10}},
        rotation=90,
        origin={0,30})));
  Modelica.Thermal.HeatTransfer.Components.ThermalConductor thermalConductor(G=
        0.59*(D^2/4*Modelica.Constants.pi)/(L2))
    "conductive heat transfer between two streams" annotation (Placement(
        transformation(
        extent={{-10,-10},{10,10}},
        rotation=90,
        origin={50,-2})));
equation
  connect(primarySupply.port_a, port_a1)
    annotation (Line(points={{-60,60},{-100,60}}, color={0,127,255}));
  connect(primarySupply.port_b, vol.ports[1])
    annotation (Line(points={{-40,60},{-2.66667,60}}, color={0,127,255}));
  connect(primaryReturn.port_a, vol1.ports[1])
    annotation (Line(points={{-40,-60},{-2.66667,-60}}, color={0,127,255}));
  connect(primaryReturn.port_b, port_b2) annotation (Line(points={{-60,-60},{-80,
          -60},{-100,-60}}, color={0,127,255}));
  connect(secondarySupply.port_a, vol.ports[2])
    annotation (Line(points={{40,60},{2.22045e-016,60}}, color={0,127,255}));
  connect(secondarySupply.port_b, port_b1)
    annotation (Line(points={{60,60},{80,60},{100,60}}, color={0,127,255}));
  connect(secondaryReturn.port_b, vol1.ports[2]) annotation (Line(points={{40,-60},
          {40,-60},{2.22045e-016,-60}}, color={0,127,255}));
  connect(secondaryReturn.port_a, port_a2)
    annotation (Line(points={{60,-60},{100,-60}}, color={0,127,255}));
  connect(tankBody.port_a, vol1.ports[3]) annotation (Line(points={{-1.77636e-015,
          -40},{-1.77636e-015,-60},{2.66667,-60}}, color={0,127,255}));
  connect(senMasFlo.port_b, vol.ports[3])
    annotation (Line(points={{0,40},{0,60},{2.66667,60}}, color={0,127,255}));
  connect(senMasFlo.port_a, tankBody.port_b)
    annotation (Line(points={{0,20},{0,0},{0,-20}}, color={0,127,255}));
  connect(thermalConductor.port_a, secondaryReturn.heatPorts[1]) annotation (
      Line(points={{50,-12},{49.9,-12},{49.9,-55.6}}, color={191,0,0}));
  connect(thermalConductor.port_b, secondarySupply.heatPorts[1]) annotation (
      Line(points={{50,8},{50,8},{50,55.6},{50.1,55.6}}, color={191,0,0}));
  annotation (Documentation(info="<html>
<h4>Overview</h4>
<p>An hydraulic separator is basically a tank or cilinder with four flanges, diametrally opposed two and two, to de-couple hydraulic systems flows from one side (e.g. primary) to the other side (e.g. secondary). </p>
<p align=\"center\"><img src=\"modelica://DCOL/Consumption/Resources/Images/hydraulicSeparator1.png\"/></p>
<p align=\"center\"><i>Schemetic of an hydraulic separator</i></p>
<p>Depending on the relation of flows between the two sides, heat and mass can be transferred from the primary to the secondary or vice-versa. </p>
<p><img src=\"modelica://DCOL/Consumption/Resources/Images/hydraulicSeparator2.png\"/><img src=\"modelica://DCOL/Consumption/Resources/Images/hydraulicSeparator3.png\"/></p>
<p><img src=\"modelica://DCOL/Consumption/Resources/Images/hydraulicSeparator4.png\"/></p>
<h4>Implementation</h4>
<p>The implementation of the hydraulic separator considers the &apos;top&apos; and &apos;bottom&apos; sections of the tank as perfectly mixed volumes with the possibility to exchange mass between them through a pipe that has the same volume as the &apos;body&apos; section. Heat can also be transferred between top and bottom by means of a thermal capacitance that has the properties of the water as medium.</p>
<p><img src=\"modelica://DCOL/Consumption/Resources/Images/hydraulicSeparator5.png\"/></p>
<h4>Notes</h4>
<p>This model is based on <a href=\"https://github.com/RWTH-EBC/AixLib/blob/master/AixLib/Fluid/MixingVolumes/HydraulicSeparator.mo\">AixLib.Fluid.MixingVolumes.HydraulicSeparator</a> with the difference that it does not assumes &apos;right&apos; design of the tank</p>
<h4>See also</h4>
<p><a href=\"https://github.com/RWTH-EBC/AixLib/blob/master/AixLib/Fluid/MixingVolumes/HydraulicSeparator.mo\">AixLib.Fluid.MixingVolumes.HydraulicSeparator</a></p>
</html>", revisions="<html>
<ul>
<li>February 20, 2017 by Raymond Sterling:<br>Implementation</li>
</ul>
</html>"), Icon(graphics={
        Polygon(
          points={{-60,100},{60,100},{60,-100},{-60,100}},
          lineColor={162,29,33},
          fillColor={162,29,33},
          fillPattern=FillPattern.Solid),
        Polygon(
          points={{-60,100},{60,100},{60,-100},{-60,100}},
          lineColor={0,0,255},
          origin={0,0},
          rotation=180,
          fillColor={0,0,255},
          fillPattern=FillPattern.Solid),
        Rectangle(
          extent={{-100,80},{-60,40}},
          lineColor={0,0,255},
          fillColor={0,0,255},
          fillPattern=FillPattern.Solid),
        Rectangle(
          extent={{-100,-40},{-60,-80}},
          lineColor={0,0,255},
          fillColor={0,0,255},
          fillPattern=FillPattern.Solid),
        Rectangle(
          extent={{60,80},{100,40}},
          lineColor={162,29,33},
          fillColor={162,29,33},
          fillPattern=FillPattern.Solid),
        Rectangle(
          extent={{60,-40},{100,-80}},
          lineColor={162,29,33},
          fillColor={162,29,33},
          fillPattern=FillPattern.Solid),
        Line(
          points={{-100,-60},{100,-60}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{-100,60},{100,60}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{-100,0},{100,0}},
          color={0,0,0},
          thickness=0.5,
          origin={0,0},
          rotation=90),
        Line(
          points={{-3,58},{3,62}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{-3,98},{3,102}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{-3,-62},{3,-58}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{-3,-102},{3,-98}},
          color={0,0,0},
          thickness=0.5),
        Text(
          extent={{-4,84},{30,72}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,255},
          fillPattern=FillPattern.Solid,
          textString="L1"),
        Text(
          extent={{-2,4},{32,-8}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,255},
          fillPattern=FillPattern.Solid,
          textString="L2"),
        Text(
          extent={{0,-76},{34,-88}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,255},
          fillPattern=FillPattern.Solid,
          textString="L3"),
        Line(
          points={{77,78},{83,82}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{77,38},{83,42}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{80,80},{80,40}},
          color={0,0,0},
          thickness=0.5),
        Text(
          extent={{46,70},{80,58}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,255},
          fillPattern=FillPattern.Solid,
          textString="Dflange"),
        Line(
          points={{57,18},{63,22}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{-63,18},{-57,22}},
          color={0,0,0},
          thickness=0.5),
        Line(
          points={{-60,20},{60,20}},
          color={0,0,0},
          thickness=0.5),
        Text(
          extent={{-10,32},{24,20}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,255},
          fillPattern=FillPattern.Solid,
          textString="D"),
        Text(
          extent={{-102,16},{-56,-16}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,255},
          fillPattern=FillPattern.Solid,
          textString="P"),
        Text(
          extent={{58,16},{104,-16}},
          lineColor={0,0,0},
          lineThickness=0.5,
          fillColor={0,0,255},
          fillPattern=FillPattern.Solid,
          textString="S")}));
end HydraulicSeparator;
