within DCOL.Distribution.Pipes.BaseClasses;
partial model PartialTwoPortPlugFlow2 "Base class for distributed flow models"
  import SI = Modelica.SIunits;
  import Modelica.Fluid.Types;
  import Modelica.Media.Interfaces.Choices.IndependentVariables;

  import Modelica.Fluid.Types.ModelStructure;

  // extending PartialTwoPort
  extends Modelica.Fluid.Interfaces.PartialTwoPort(
    final port_a_exposesState = (modelStructure == ModelStructure.av_b) or (modelStructure == ModelStructure.av_vb),
    final port_b_exposesState = (modelStructure == ModelStructure.a_vb) or (modelStructure == ModelStructure.av_vb));

  // distributed volume model
  extends Modelica.Fluid.Interfaces.PartialDistributedVolume(
    final energyDynamics = Types.Dynamics.DynamicFreeInitial,
    final massDynamics = Types.Dynamics.SteadyState,
    final n = nNodes,
    final fluidVolumes = (crossAreas.*lengths)*nParallel);

  // Geometry parameters
  parameter Real nParallel(min=1)=1
    "Number of identical parallel flow devices"
    annotation(Dialog(group="Geometry"));
  parameter SI.Length[n] lengths "lengths of flow segments"
    annotation(Dialog(group="Geometry"));
  parameter SI.Area[n] crossAreas "cross flow areas of flow segments"
    annotation(Dialog(group="Geometry"));
  parameter SI.Length[n] dimensions "hydraulic diameters of flow segments"
    annotation(Dialog(group="Geometry"));
  parameter SI.Height[n] roughnesses
    "Average heights of surface asperities"
    annotation(Dialog(group="Geometry"));

  // Static head
  parameter SI.Length[n] dheights=zeros(n)
    "Differences in heights of flow segments"
      annotation(Dialog(group="Static head"), Evaluate=true);

  // Assumptions
  parameter Types.Dynamics momentumDynamics=system.momentumDynamics
    "Formulation of momentum balances"
    annotation(Evaluate=true, Dialog(tab = "Assumptions", group="Dynamics"));

  // Initialization
  parameter Medium.MassFlowRate m_flow_start = system.m_flow_start
    "Start value for mass flow rate"
     annotation(Evaluate=true, Dialog(tab = "Initialization"));

  // Discretization
  parameter Integer nNodes(min=1)=2 "Number of discrete flow volumes"
    annotation(Dialog(tab="Advanced"),Evaluate=true);

  parameter Types.ModelStructure modelStructure=Types.ModelStructure.av_vb
    "Determines whether flow or volume models are present at the ports"
    annotation(Dialog(tab="Advanced"), Evaluate=true);

  parameter Boolean useLumpedPressure=false
    "=true to lump pressure states together"
    annotation(Dialog(tab="Advanced"),Evaluate=true);
  final parameter Integer nFM=if useLumpedPressure then nFMLumped else nFMDistributed
    "number of flow models in flowModel";
  final parameter Integer nFMDistributed=if modelStructure==Types.ModelStructure.a_v_b then n+1 else if (modelStructure==Types.ModelStructure.a_vb or modelStructure==Types.ModelStructure.av_b) then n else n-1;
  final parameter Integer nFMLumped=if modelStructure==Types.ModelStructure.a_v_b then 2 else 1;
  final parameter Integer iLumped=integer(n/2)+1
    "Index of control volume with representative state if useLumpedPressure"
    annotation(Evaluate=true);

  // Advanced model options
  parameter Boolean useInnerPortProperties=false
    "=true to take port properties for flow models from internal control volumes"
    annotation(Dialog(tab="Advanced"),Evaluate=true);
  Medium.ThermodynamicState state_a
    "state defined by volume outside port_a";
  Medium.ThermodynamicState state_b
    "state defined by volume outside port_b";
  Medium.ThermodynamicState[nFM+1] statesFM
    "state vector for flowModel model";

  // Pressure loss model
  replaceable model FlowModel =
    Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow
    constrainedby Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel
    "Wall friction, gravity, momentum flow"
      annotation(Dialog(group="Pressure loss"), choicesAllMatching=true);
  FlowModel flowModel(
          redeclare final package Medium = Medium,
          final n=nFM+1,
          final states=statesFM,
          final vs=vsFM,
          final momentumDynamics=momentumDynamics,
          final allowFlowReversal=allowFlowReversal,
          final p_a_start=p_a_start,
          final p_b_start=p_b_start,
          final m_flow_start=m_flow_start,
          final nParallel=nParallel,
          final pathLengths=pathLengths,
          final crossAreas=crossAreasFM,
          final dimensions=dimensionsFM,
          final roughnesses=roughnessesFM,
          final dheights=dheightsFM,
          final g=system.g) "Flow model"
     annotation (Placement(transformation(extent={{-77,-37},{75,-19}})));

  // Flow quantities
  Medium.MassFlowRate[n+1] m_flows(
     each min=if allowFlowReversal then -Modelica.Constants.inf else 0,
     each start=m_flow_start)
    "Mass flow rates of fluid across segment boundaries";
  // Medium.MassFlowRate[n+1, Medium.nXi] mXi_flows
  //   "Independent mass flow rates across segment boundaries";
  // Medium.MassFlowRate[n+1, Medium.nC] mC_flows
  //   "Trace substance mass flow rates across segment boundaries";
  // Medium.EnthalpyFlowRate[n+1] H_flows
  //   "Enthalpy flow rates of fluid across segment boundaries";

  SI.Velocity[n] vs = {0.5*(m_flows[i] + m_flows[i+1])/mediums[i].d/crossAreas[i] for i in 1:n}/nParallel
    "mean velocities in flow segments";

  // Model structure dependent flow geometry
protected
  SI.Length[nFM] pathLengths "Lengths along flow path";
  SI.Length[nFM] dheightsFM "Differences in heights between flow segments";
  SI.Area[nFM+1] crossAreasFM "Cross flow areas of flow segments";
  SI.Velocity[nFM+1] vsFM "Mean velocities in flow segments";
  SI.Length[nFM+1] dimensionsFM "Hydraulic diameters of flow segments";
  SI.Height[nFM+1] roughnessesFM "Average heights of surface asperities";

public
  constant Boolean use_h = Medium.ThermoStates == IndependentVariables.ph or
                           Medium.ThermoStates == IndependentVariables.phX "Use specific enthalpy as state variable";

  // INPUTS for plug-flow model
  SI.Enthalpy H_aIN;
  SI.Enthalpy H_bIN;
  SI.Mass[Medium.nXi] mXi_aIN;
  SI.Mass[Medium.nXi] mXi_bIN;
  SI.Mass[Medium.nC] mC_aIN;
  SI.Mass[Medium.nC] mC_bIN;
  Medium.BaseProperties medium_aIN(
    preferredMediumStates=true,
    p(start=p_a_start),
    h(start=h_start),
    T(start=T_start),
    Xi(start=X_start[1:Medium.nXi]));
  Medium.BaseProperties medium_bIN(
    preferredMediumStates=true,
    p(start=p_a_start),
    h(start=h_start),
    T(start=T_start),
    Xi(start=X_start[1:Medium.nXi]));
  Medium.ExtraProperty[Medium.nC] C_aIN;
  Medium.ExtraProperty[Medium.nC] C_bIN;

  // OUTPUTS for plug-flow model
  Medium.BaseProperties medium_aOUT(
    preferredMediumStates=true,
    p(start=p_a_start),
    h(start=h_start),
    T(start=T_start),
    Xi(start=X_start[1:Medium.nXi]));
  Medium.BaseProperties medium_bOUT(
    preferredMediumStates=true,
    p(start=p_b_start),
    h(start=h_start),
    T(start=T_start),
    Xi(start=X_start[1:Medium.nXi]));
  Medium.ExtraProperty[Medium.nC] C_aOUT;
  Medium.ExtraProperty[Medium.nC] C_bOUT;

  // Variables used to generate flow-plug events
  SI.Velocity v_avg "Fluid average velocity";
  SI.Length x(start=0, each stateSelect = StateSelect.always) "Distant travelled by the flow-plug segment";

  // States variables
  Medium.Temperature[n] Ts(
    each stateSelect = (if use_h then StateSelect.never else StateSelect.always));
  Medium.SpecificEnthalpy[n] hs(
    each stateSelect = (if use_h then StateSelect.always else StateSelect.never));
  Medium.MassFraction[n, Medium.nXi] Xis;
  //Cs from Modelica.Fluid.Interfaces.PartialDistributedVolume

equation
  // State variables
  Ts = mediums.T;
  hs = mediums.h;
  Xis = {mediums[i].Xi for i in 1:n};

  // Boundary conditions for plug flow (as inputs)
  medium_aIN.p = mediums[1].p;//port_a.p;
  medium_bIN.p = mediums[n].p;//port_b.p;
  medium_aIN.h = max(H_aIN/ms[1], 0);
  medium_bIN.h = max(H_bIN/ms[n], 0);
  medium_aIN.Xi = mXi_aIN/ms[1];
  medium_bIN.Xi = mXi_bIN/ms[n];
  C_aIN = mC_aIN/ms[1];
  C_bIN = mC_bIN/ms[n];
  // Transport equations at boundary mediums
  der(H_aIN) = port_a.m_flow * actualStream(port_a.h_outflow);
  der(H_bIN) = port_b.m_flow * actualStream(port_b.h_outflow);
  der(mXi_aIN) = port_a.m_flow * actualStream(port_a.Xi_outflow);
  der(mXi_bIN) = port_b.m_flow * actualStream(port_b.Xi_outflow);
  der(mC_aIN) = port_a.m_flow * actualStream(port_a.C_outflow);
  der(mC_bIN) = port_b.m_flow * actualStream(port_b.C_outflow);

  // Boundary conditions (as outputs)
  // port_a.p = medium_aOUT.p +
  //   (if modelStructure==ModelStructure.a_v_b or modelStructure==ModelStructure.a_vb
  //   then flowModel.dps_fg[1]
  //   else 0);
  // port_b.p = medium_bOUT.p +
  //   (if modelStructure==ModelStructure.a_v_b or modelStructure==ModelStructure.av_b
  //   then flowModel.dps_fg[end]
  //   else 0);
  port_a.p = medium_aOUT.p;
  port_b.p = medium_bOUT.p;
  port_a.h_outflow = medium_aOUT.h;
  port_b.h_outflow = medium_bOUT.h;
  port_a.Xi_outflow = medium_aOUT.Xi;
  port_b.Xi_outflow = medium_bOUT.Xi;
  port_a.C_outflow = C_aOUT;
  port_a.C_outflow = C_bOUT;
  // No change in the transported variables at boundary mediums
  if use_h then
    der(medium_aOUT.h) = 0;
    der(medium_bOUT.h) = 0;
  else
    der(medium_aOUT.T) = 0;
    der(medium_bOUT.T) = 0;
  end if;
  der(medium_aOUT.Xi) = zeros(Medium.nXi);
  der(medium_bOUT.Xi) = zeros(Medium.nXi);
  der(C_aOUT) = zeros(Medium.nC);
  der(C_bOUT) = zeros(Medium.nC);

  // Source/sink terms for mass and energy balances
  mb_flows = m_flows[1:n] - m_flows[2:n+1];
  mbXi_flows = fill(zeros(Medium.nXi), n); //zeros due to the plug flow approach
  mbC_flows  = fill(zeros(Medium.nC), n); //zeros due to the plug flow approach
  Hb_flows = zeros(n); //zeros due to the plug flow approach

  // Plug flow discretization
  v_avg = sum(vs)/n;
  der(x) = v_avg;
  when {x>=lengths[1], x<=-lengths[n]} then
    if use_h then
      reinit(medium_aOUT.h, pre(hs[1]));
      reinit(medium_bOUT.h, pre(hs[n]));
      reinit(hs, (if v_avg>0 then
        cat(1, {medium_aIN.h}, pre(hs[1:n-1])) else
        cat(1, pre(hs[2:n]), {medium_bIN.h})));
    else
      reinit(medium_aOUT.T, pre(Ts[1]));
      reinit(medium_bOUT.T, pre(Ts[n]));
      reinit(Ts, (if v_avg>0 then
        cat(1, {medium_aIN.T}, pre(Ts[1:n-1])) else
        cat(1, pre(Ts[2:n]), {medium_bIN.T})));
    end if;
    if Medium.nXi>0 then
      reinit(medium_aOUT.Xi, pre(Xis[1,:]));
      reinit(medium_bOUT.Xi, pre(Xis[n,:]));
      reinit(Xis, if v_avg>0 then
        cat(1, {medium_aIN.Xi}, pre(Xis[1:n-1,:])) else
        cat(1, pre(Xis[2:n,:]), {medium_bIN.Xi}));
    end if;
    if Medium.nC>0 then
      reinit(C_aOUT, pre(Cs[1,:]));
      reinit(C_bOUT, pre(Cs[n,:]));
      reinit(Cs, if v_avg>0 then
        cat(1, {C_aIN}, pre(Cs[1:n-1,:])) else
        cat(1, pre(Cs[2:n,:]), {C_bIN}));
    end if;
  end when;
  when {x>=lengths[1], x<=-lengths[n], v_avg>0, v_avg<0} then
    reinit(x, 0);
    reinit(H_aIN, 0);
    reinit(H_bIN, 0);
    if Medium.nXi>0 then
      reinit(mXi_aIN, zeros(Medium.nXi));
      reinit(mXi_bIN, zeros(Medium.nXi));
    end if;
    if Medium.nC>0 then
      reinit(mC_aIN, zeros(Medium.nC));
      reinit(mC_bIN, zeros(Medium.nC));
    end if;
  end when;

  // SAME AS PartialTwoPortFlow ******************************************************************* //
  assert(nNodes > 1 or modelStructure <> ModelStructure.av_vb,
     "nNodes needs to be at least 2 for modelStructure av_vb, as flow model disappears otherwise!");
  // staggered grid discretization of geometry for flowModel, depending on modelStructure
  if useLumpedPressure then
    if modelStructure <> ModelStructure.a_v_b then
      pathLengths[1] = sum(lengths);
      dheightsFM[1] = sum(dheights);
      if n == 1 then
        crossAreasFM[1:2] = {crossAreas[1], crossAreas[1]};
        dimensionsFM[1:2] = {dimensions[1], dimensions[1]};
        roughnessesFM[1:2] = {roughnesses[1], roughnesses[1]};
      else // n > 1
        crossAreasFM[1:2] = {sum(crossAreas[1:iLumped-1])/(iLumped-1), sum(crossAreas[iLumped:n])/(n-iLumped+1)};
        dimensionsFM[1:2] = {sum(dimensions[1:iLumped-1])/(iLumped-1), sum(dimensions[iLumped:n])/(n-iLumped+1)};
        roughnessesFM[1:2] = {sum(roughnesses[1:iLumped-1])/(iLumped-1), sum(roughnesses[iLumped:n])/(n-iLumped+1)};
      end if;
    else
      if n == 1 then
        pathLengths[1:2] = {lengths[1]/2, lengths[1]/2};
        dheightsFM[1:2] = {dheights[1]/2, dheights[1]/2};
        crossAreasFM[1:3] = {crossAreas[1], crossAreas[1], crossAreas[1]};
        dimensionsFM[1:3] = {dimensions[1], dimensions[1], dimensions[1]};
        roughnessesFM[1:3] = {roughnesses[1], roughnesses[1], roughnesses[1]};
      else // n > 1
        pathLengths[1:2] = {sum(lengths[1:iLumped-1]), sum(lengths[iLumped:n])};
        dheightsFM[1:2] = {sum(dheights[1:iLumped-1]), sum(dheights[iLumped:n])};
        crossAreasFM[1:3] = {sum(crossAreas[1:iLumped-1])/(iLumped-1), sum(crossAreas)/n, sum(crossAreas[iLumped:n])/(n-iLumped+1)};
        dimensionsFM[1:3] = {sum(dimensions[1:iLumped-1])/(iLumped-1), sum(dimensions)/n, sum(dimensions[iLumped:n])/(n-iLumped+1)};
        roughnessesFM[1:3] = {sum(roughnesses[1:iLumped-1])/(iLumped-1), sum(roughnesses)/n, sum(roughnesses[iLumped:n])/(n-iLumped+1)};
      end if;
    end if;
  else
    if modelStructure == ModelStructure.av_vb then
      //nFM = n-1
      if n == 2 then
        pathLengths[1] = lengths[1] + lengths[2];
        dheightsFM[1] = dheights[1] + dheights[2];
      else
        pathLengths[1:n-1] = cat(1, {lengths[1] + 0.5*lengths[2]}, 0.5*(lengths[2:n-2] + lengths[3:n-1]), {0.5*lengths[n-1] + lengths[n]});
        dheightsFM[1:n-1] = cat(1, {dheights[1] + 0.5*dheights[2]}, 0.5*(dheights[2:n-2] + dheights[3:n-1]), {0.5*dheights[n-1] + dheights[n]});
      end if;
      crossAreasFM[1:n] = crossAreas;
      dimensionsFM[1:n] = dimensions;
      roughnessesFM[1:n] = roughnesses;
    elseif modelStructure == ModelStructure.av_b then
      //nFM = n
      pathLengths[1:n] = lengths;
      dheightsFM[1:n] = dheights;
      crossAreasFM[1:n+1] = cat(1, crossAreas[1:n], {crossAreas[n]});
      dimensionsFM[1:n+1] = cat(1, dimensions[1:n], {dimensions[n]});
      roughnessesFM[1:n+1] = cat(1, roughnesses[1:n], {roughnesses[n]});
    elseif modelStructure == ModelStructure.a_vb then
      //nFM = n
      pathLengths[1:n] = lengths;
      dheightsFM[1:n] = dheights;
      crossAreasFM[1:n+1] = cat(1, {crossAreas[1]}, crossAreas[1:n]);
      dimensionsFM[1:n+1] = cat(1, {dimensions[1]}, dimensions[1:n]);
      roughnessesFM[1:n+1] = cat(1, {roughnesses[1]}, roughnesses[1:n]);
    elseif modelStructure == ModelStructure.a_v_b then
      //nFM = n+1;
      pathLengths[1:n+1] = cat(1, {0.5*lengths[1]}, 0.5*(lengths[1:n-1] + lengths[2:n]), {0.5*lengths[n]});
      dheightsFM[1:n+1] = cat(1, {0.5*dheights[1]}, 0.5*(dheights[1:n-1] + dheights[2:n]), {0.5*dheights[n]});
      crossAreasFM[1:n+2] = cat(1, {crossAreas[1]}, crossAreas[1:n], {crossAreas[n]});
      dimensionsFM[1:n+2] = cat(1, {dimensions[1]}, dimensions[1:n], {dimensions[n]});
      roughnessesFM[1:n+2] = cat(1, {roughnesses[1]}, roughnesses[1:n], {roughnesses[n]});
    else
      assert(false, "Unknown model structure");
    end if;
  end if;

  // Source/sink terms for mass and energy balances
  // for i in 1:n loop
  //   mb_flows[i] = m_flows[i] - m_flows[i + 1];
  //   mbXi_flows[i, :] = mXi_flows[i, :] - mXi_flows[i + 1, :];
  //   mbC_flows[i, :]  = mC_flows[i, :]  - mC_flows[i + 1, :];
  //   Hb_flows[i] = H_flows[i] - H_flows[i + 1];
  // end for;

  // // Distributed flow quantities, upwind discretization
  // for i in 2:n loop
  //   H_flows[i] = semiLinear(m_flows[i], mediums[i - 1].h, mediums[i].h);
  //   mXi_flows[i, :] = semiLinear(m_flows[i], mediums[i - 1].Xi, mediums[i].Xi);
  //   mC_flows[i, :]  = semiLinear(m_flows[i], Cs[i - 1, :],         Cs[i, :]);
  // end for;
  // H_flows[1] = semiLinear(port_a.m_flow, inStream(port_a.h_outflow), mediums[1].h);
  // H_flows[n + 1] = -semiLinear(port_b.m_flow, inStream(port_b.h_outflow), mediums[n].h);
  // mXi_flows[1, :] = semiLinear(port_a.m_flow, inStream(port_a.Xi_outflow), mediums[1].Xi);
  // mXi_flows[n + 1, :] = -semiLinear(port_b.m_flow, inStream(port_b.Xi_outflow), mediums[n].Xi);
  // mC_flows[1, :] = semiLinear(port_a.m_flow, inStream(port_a.C_outflow), Cs[1, :]);
  // mC_flows[n + 1, :] = -semiLinear(port_b.m_flow, inStream(port_b.C_outflow), Cs[n, :]);

  // Boundary conditions
  port_a.m_flow = m_flows[1];
  port_b.m_flow = -m_flows[n + 1];
  // port_a.h_outflow = mediums[1].h;
  // port_b.h_outflow = mediums[n].h;
  // port_a.Xi_outflow = mediums[1].Xi;
  // port_b.Xi_outflow = mediums[n].Xi;
  // port_a.C_outflow = Cs[1, :];
  // port_b.C_outflow = Cs[n, :];
  // The two equations below are not correct if C is stored in volumes.
  // C should be treated the same way as Xi.
  //port_a.C_outflow = inStream(port_b.C_outflow);
  //port_b.C_outflow = inStream(port_a.C_outflow);

  if useInnerPortProperties and n > 0 then
    state_a = Medium.setState_phX(port_a.p, mediums[1].h, mediums[1].Xi);
    state_b = Medium.setState_phX(port_b.p, mediums[n].h, mediums[n].Xi);
  else
    state_a = Medium.setState_phX(port_a.p, inStream(port_a.h_outflow), inStream(port_a.Xi_outflow));
    state_b = Medium.setState_phX(port_b.p, inStream(port_b.h_outflow), inStream(port_b.Xi_outflow));
  end if;

  // staggered grid discretization for flowModel, depending on modelStructure
  if useLumpedPressure then
    if modelStructure <> ModelStructure.av_vb then
      // all pressures are equal
      fill(mediums[1].p, n-1) = mediums[2:n].p;
    elseif n > 2 then
      // need two pressures
      fill(mediums[1].p, iLumped-2) = mediums[2:iLumped-1].p;
      fill(mediums[n].p, n-iLumped) = mediums[iLumped:n-1].p;
    end if;
    if modelStructure == ModelStructure.av_vb then
      port_a.p = mediums[1].p;
      statesFM[1] = mediums[1].state;
      m_flows[iLumped] = flowModel.m_flows[1];
      statesFM[2] = mediums[n].state;
      port_b.p = mediums[n].p;
      vsFM[1] = vs[1:iLumped-1]*lengths[1:iLumped-1]/sum(lengths[1:iLumped-1]);
      vsFM[2] = vs[iLumped:n]*lengths[iLumped:n]/sum(lengths[iLumped:n]);
    elseif modelStructure == ModelStructure.av_b then
      port_a.p = mediums[1].p;
      statesFM[1] = mediums[iLumped].state;
      statesFM[2] = state_b;
      m_flows[n+1] = flowModel.m_flows[1];
      vsFM[1] = vs*lengths/sum(lengths);
      vsFM[2] = m_flows[n+1]/Medium.density(state_b)/crossAreas[n]/nParallel;
    elseif modelStructure == ModelStructure.a_vb then
      m_flows[1] = flowModel.m_flows[1];
      statesFM[1] = state_a;
      statesFM[2] = mediums[iLumped].state;
      port_b.p = mediums[n].p;
      vsFM[1] = m_flows[1]/Medium.density(state_a)/crossAreas[1]/nParallel;
      vsFM[2] = vs*lengths/sum(lengths);
    elseif modelStructure == ModelStructure.a_v_b then
      m_flows[1] = flowModel.m_flows[1];
      statesFM[1] = state_a;
      statesFM[2] = mediums[iLumped].state;
      statesFM[3] = state_b;
      m_flows[n+1] = flowModel.m_flows[2];
      vsFM[1] = m_flows[1]/Medium.density(state_a)/crossAreas[1]/nParallel;
      vsFM[2] = vs*lengths/sum(lengths);
      vsFM[3] = m_flows[n+1]/Medium.density(state_b)/crossAreas[n]/nParallel;
    else
      assert(false, "Unknown model structure");
    end if;
  else
    if modelStructure == ModelStructure.av_vb then
      //nFM = n-1
      statesFM[1:n] = mediums[1:n].state;
      m_flows[2:n] = flowModel.m_flows[1:n-1];
      vsFM[1:n] = vs;
      port_a.p = mediums[1].p;
      port_b.p = mediums[n].p;
    elseif modelStructure == ModelStructure.av_b then
      //nFM = n
      statesFM[1:n] = mediums[1:n].state;
      statesFM[n+1] = state_b;
      m_flows[2:n+1] = flowModel.m_flows[1:n];
      vsFM[1:n] = vs;
      vsFM[n+1] = m_flows[n+1]/Medium.density(state_b)/crossAreas[n]/nParallel;
      port_a.p = mediums[1].p;
    elseif modelStructure == ModelStructure.a_vb then
      //nFM = n
      statesFM[1] = state_a;
      statesFM[2:n+1] = mediums[1:n].state;
      m_flows[1:n] = flowModel.m_flows[1:n];
      vsFM[1] = m_flows[1]/Medium.density(state_a)/crossAreas[1]/nParallel;
      vsFM[2:n+1] = vs;
      port_b.p = mediums[n].p;
    elseif modelStructure == ModelStructure.a_v_b then
      //nFM = n+1
      statesFM[1] = state_a;
      statesFM[2:n+1] = mediums[1:n].state;
      statesFM[n+2] = state_b;
      m_flows[1:n+1] = flowModel.m_flows[1:n+1];
      vsFM[1] = m_flows[1]/Medium.density(state_a)/crossAreas[1]/nParallel;
      vsFM[2:n+1] = vs;
      vsFM[n+2] = m_flows[n+1]/Medium.density(state_b)/crossAreas[n]/nParallel;
    else
      assert(false, "Unknown model structure");
    end if;
  end if;

  annotation (defaultComponentName="pipe",
Documentation(info="<html>
<p>Base class for distributed flow models based on the plug-flow approach. In order to make the model compatible with any model based on <a href=\"Modelica.Fluid\">Modelica.Fluid</a>, the structure and most of the equations are taken from <a href=\"Modelica.Fluid.Pipes.BaseClasses.PartialTwoPortFlow\">Modelica.Fluid.Pipes.BaseClasses.PartialTwoPortFlow</a>. Some of the original equations were modified and new equations added to substitute the upwind discretization by the plug-flow method.</p>
<p>The total volume is split into nNodes segments along the flow path. The default value is nNodes=2. </p>
<p><b>Avoiding the artificial diffusion and a new pipe model</b> </p>
<p>One of the main issues encountered when using numerical schemes to solve differential equations is the introduction of artificial (also called numerical) diffusion. In particular, the problem becomes noticeable in computer simulations of continua (such as fluids) based on the upwind-difference scheme wherein the simulated medium exhibits a higher diffusivity (the ability regarding transport phenomena, heat transport, mass transport or momentum transport among others) than the true medium [1]. &nbsp; </p>
<p>One way to minimise the artificial diffusion is to increase the degree of geometrical discretisation of the element under study, for this research work, the pipe. However, in simulations of district heating and cooling where pipes are usually too long when compared to the distance travelled by the fluid during the simulation time step, this solution becomes non feasible since a high degree of discretisation leads to a number of equations unmanageable for most of the simulation software and hardware. Furthermore, to keep the numerical algorithm stable the time step has to obey the Courant-Friedrichs-Lewy condition [2], in consequence, the time step must be reduced, which increases the computational cost of the simulation too; and not all of the simulation solvers allow the user to manipulate the time step. &nbsp; </p>
<p>[3]proposed a Modelica model of the pipe in which the energy balance equation is derived according to the method of characteristics [4]. Following this method and without taken into account the heat capacity of the tube material, the partial differential equation becomes an ordinary differential equation, natively integrated using any implementation of Modelica. The outlet temperature obtained from this integration is then modified to account for the heat stored in the tube material by assuming that the heat capacity of the whole tube is concentrated at the outlet of the pipe and that the fluid and tube are at the same temperature [5]. This approach speeds up the simulations and reduces the artificial diffusion noticeably. However, due to the later assumption, there is still a considerable error in the calculation of the heat losses, which becomes more relevant in simulations of district cooling. &nbsp; </p>
<p>This pipe model is based on the Type31 model of TRNSYS. Based on the plug-flow approach, the Type31 model represents the thermal behaviour of a fluid flowing through a pipe using variable size segments of fluid. Entering fluid shifts the position of existing segments. The mass of the entering segment is calculated by multiplying the mass flow rate by the simulation time step. The incoming fluid temperature defines de temperature of the new segment. Finally, the outlet temperature is the mass weighted average of the segments that are pushed out by the inlet flow [7]. &nbsp; </p>
<p>The TRANSYS model of the pipe has limitations due to some assumption it makes. On the one hand, the energy losses equation uses a constant overall loss conductance, which does not include the convective effect neither the heat capacity of the pipe material. On the other hand, the external temperature along the entire pipe is the same i.e. no external temperature profile may be input into the model. These assumptions makes the model unsuitable for some applications, such as Model Predictive Control for heating and cooling district where the dynamics of the piping system is taken into account take advantage of the energy the pipe may store and the response delay due to long pipes</p>
<p>Consequently, it is required a detailed model that considers the dynamics of the pipe in detail, and all its inputs may be geometrically discretised in the length direction without introducing artificial diffusion. As mentioned above, TRANSYS model does not satisfy these requirements. That is why this model, tough based on the flow plug idea of TRNSYS model, introduces the following modifications: </p>
<p style=\"margin-left: 30px;\"><u>External temperature profile</u>. The new model allows to input an external temperature profile which matches to the geometrical discretisation of the tube. </p>
<p style=\"margin-left: 30px;\"><u>Fixed size of segments</u>. Since Modelica allows the use of simulation events, every new segment is pulled in only when it has the size defined by a parameter of the model. This feature eases the matching of the discretisation of the pipe and the corresponding external temperature profile. </p>
<p style=\"margin-left: 30px;\"><u>Convective heat transfer coefficient</u>. The heat transfer model used by the new pipe model takes into account the convective effect. </p>
<p style=\"margin-left: 30px;\"><u>Heat capacity of the pipe</u>. The new model includes the influence of the thermal capacity of the pipe material and not just the thermal conductance effect. </p>
<h4>Mass and Energy balances</h4>
<p>The mass and energy balances are inherited from <a href=\"modelica://Modelica.Fluid.Interfaces.PartialDistributedVolume\">Interfaces.PartialDistributedVolume</a>. One total mass and one energy balance is formed across each segment according to the finite volume approach. Substance mass balances are added if the medium contains more than one component. </p>
<p>An extending model needs to define the geometry and the difference in heights between the flow segments (static head). Moreover it needs to define two vectors of source terms for the distributed energy balance: </p>
<ul>
<li><code><b>Qb_flows[nNodes]</b></code>, the heat flow source terms, e.g., conductive heat flows across segment boundaries, and</li>
<li><code><b>Wb_flows[nNodes]</b></code>, the work source terms. </li>
</ul>
<h4>Momentum balance</h4>
<p>The momentum balance is determined by the <code><b>FlowModel</b></code> component, which can be replaced with any model extended from <a href=\"modelica://Modelica.Fluid.Pipes.BaseClasses.FlowModels.PartialStaggeredFlowModel\">BaseClasses.FlowModels.PartialStaggeredFlowModel</a>. The default setting is <a href=\"modelica://Modelica.Fluid.Pipes.BaseClasses.FlowModels.DetailedPipeFlow\">DetailedPipeFlow</a>. </p>
<p>This considers </p>
<ul>
<li>pressure drop due to friction and other dissipative losses, and</li>
<li>gravity effects for non-horizontal devices.</li>
<li>variation of flow velocity along the flow path, which occur due to changes in the cross sectional area or the fluid density, provided that <code>flowModel.use_Ib_flows</code> is true. </li>
</ul>
<h4>Model Structure</h4>
<p>The momentum balances are formulated across the segment boundaries along the flow path according to the staggered grid approach. The configurable <code><b>modelStructure</b></code> determines the formulation of the boundary conditions at <code>port_a</code> and <code>port_b</code>. The options include (default: av_vb): </p>
<ul>
<li><code>av_vb</code>: Symmetric setting with nNodes-1 momentum balances between nNodes flow segments. The ports <code>port_a</code> and <code>port_b</code> expose the first and the last thermodynamic state, respectively. Connecting two or more flow devices therefore may result in high-index DAEs for the pressures of connected flow segments. </li>
<li><code>a_v_b</code>: Alternative symmetric setting with nNodes+1 momentum balances across nNodes flow segments. Half momentum balances are placed between <code>port_a</code> and the first flow segment as well as between the last flow segment and <code>port_b</code>. Connecting two or more flow devices therefore results in algebraic pressures at the ports. The specification of good start values for the port pressures is essential for the solution of large nonlinear equation systems.</li>
<li><code>av_b</code>: Asymmetric setting with nNodes momentum balances, one between nth volume and <code>port_b</code>, potential pressure state at <code>port_a</code></li>
<li><code>a_vb</code>: Asymmetric setting with nNodes momentum balance, one between first volume and <code>port_a</code>, potential pressure state at <code>port_b</code></li>
</ul>
<p>When connecting two components, e.g., two pipes, the momentum balance across the connection point reduces to </p>
<pre>pipe1.port_b.p = pipe2.port_a.p</pre>
<p>This is only true if the flow velocity remains the same on each side of the connection. Consider using a fitting for any significant change in diameter or fluid density, if the resulting effects, such as change in kinetic energy, cannot be neglected. This also allows for taking into account friction losses with respect to the actual geometry of the connection point. </p>
<h4>References</h4>
<p>[1]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; Leonard BP. A stable and accurate convective modelling procedure based on quadratic upstream interpolation. Comput Methods Appl Mech Eng 1979;19:59&ndash;98. doi:10.1016/0045-7825(79)90034-3. </p>
<p>[2]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; Courant R, Friedrichs K, Lewy H. On the partial difference equations of mathematical physics. IBM J Res Dev 1967;2:215&ndash;34. </p>
<p>[3]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; Giraud L, Bavi&egrave;re R, Paulus C, Vall&eacute;e M, Robin J. Dynamic Modelling , Experimental Validation and Simulation of a Virtual District Heating Network 2015. </p>
<p>[4]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; Wylie EB, Streeter VL. Fluid Transients. McGraw-Hill International; 1978. </p>
<p>[5]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; Giraud L, Bavi&egrave;re R, Vall&eacute;e M, Paulus C. Presentation, Validation and Application of the DistrictHeating Modelica Library. Proc 11th Model Conf 2015:79&ndash;88. doi:10.3384/ecp1511879. </p>
<p>[6]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; INDIGO consortium. INDIGO project 2016. http://www.indigo-project.eu/ (accessed January 25, 2017). </p>
<p>[7]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; Solar Energy Laboratory. Trnsys 17 Mathematical Reference. vol. 4. Wisconsin, Madison: 2009. </p>
</html>",
    revisions="<html>
<ul>
<li></li>
</ul>
</html>"),
Icon(coordinateSystem(preserveAspectRatio=true,  extent={{-100,-100},{100,
            100}}), graphics={Ellipse(
          extent={{-72,10},{-52,-10}},
          lineColor={0,0,0},
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid), Ellipse(
          extent={{50,10},{70,-10}},
          lineColor={0,0,0},
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid)}),
Diagram(coordinateSystem(preserveAspectRatio=true,  extent={{-100,-100},{
            100,100}}), graphics={
        Polygon(
          points={{-100,-50},{-100,50},{100,60},{100,-60},{-100,-50}},
          fillColor={215,215,215},
          fillPattern=FillPattern.Solid,
          pattern=LinePattern.None,
          lineColor={0,0,0}),
        Polygon(
          points={{-34,-53},{-34,53},{34,57},{34,-57},{-34,-53}},
          fillColor={255,255,255},
          fillPattern=FillPattern.Solid,
          pattern=LinePattern.None,
          lineColor={0,0,0}),
        Line(
          points={{-100,-50},{-100,50}},
          arrow={Arrow.Filled,Arrow.Filled},
          pattern=LinePattern.Dot),
        Text(
          extent={{-99,36},{-69,30}},
          lineColor={0,0,255},
          textString="crossAreas[1]"),
        Line(
          points={{-100,70},{-34,70}},
          arrow={Arrow.Filled,Arrow.Filled},
          pattern=LinePattern.Dot),
        Text(
          extent={{0,36},{40,30}},
          lineColor={0,0,255},
          textString="crossAreas[2:n-1]"),
        Line(
          points={{100,-60},{100,60}},
          arrow={Arrow.Filled,Arrow.Filled},
          pattern=LinePattern.Dot),
        Text(
          extent={{100.5,36},{130.5,30}},
          lineColor={0,0,255},
          textString="crossAreas[n]"),
        Line(
          points={{-34,52},{-34,-53}},
          pattern=LinePattern.Dash),
        Line(
          points={{34,57},{34,-57}},
          pattern=LinePattern.Dash),
        Line(
          points={{34,70},{100,70}},
          arrow={Arrow.Filled,Arrow.Filled},
          pattern=LinePattern.Dot),
        Line(
          points={{-34,70},{34,70}},
          arrow={Arrow.Filled,Arrow.Filled},
          pattern=LinePattern.Dot),
        Text(
          extent={{-30,77},{30,71}},
          lineColor={0,0,255},
          textString="lengths[2:n-1]"),
        Line(
          points={{-100,-70},{0,-70}},
          arrow={Arrow.None,Arrow.Filled}),
        Text(
          extent={{-80,-63},{-20,-69}},
          lineColor={0,0,255},
          textString="flowModel.dps_fg[1]"),
        Line(
          points={{0,-70},{100,-70}},
          arrow={Arrow.None,Arrow.Filled}),
        Text(
          extent={{20.5,-63},{80,-69}},
          lineColor={0,0,255},
          textString="flowModel.dps_fg[2:n-1]"),
        Line(
          points={{-95,0},{-5,0}},
          arrow={Arrow.None,Arrow.Filled}),
        Text(
          extent={{-62,7},{-32,1}},
          lineColor={0,0,255},
          textString="m_flows[2]"),
        Line(
          points={{5,0},{95,0}},
          arrow={Arrow.None,Arrow.Filled}),
        Text(
          extent={{34,7},{64,1}},
          lineColor={0,0,255},
          textString="m_flows[3:n]"),
        Line(
          points={{-150,0},{-105,0}},
          arrow={Arrow.None,Arrow.Filled}),
        Line(
          points={{105,0},{150,0}},
          arrow={Arrow.None,Arrow.Filled}),
        Text(
          extent={{-140,7},{-110,1}},
          lineColor={0,0,255},
          textString="m_flows[1]"),
        Text(
          extent={{111,7},{141,1}},
          lineColor={0,0,255},
          textString="m_flows[n+1]"),
        Text(
          extent={{35,-92},{100,-98}},
          lineColor={0,0,255},
          textString="(ModelStructure av_vb, n=3)"),
        Line(
          points={{-100,-50},{-100,-86}},
          pattern=LinePattern.Dot),
        Line(
          points={{0,-55},{0,-86}},
          pattern=LinePattern.Dot),
        Line(
          points={{100,-60},{100,-86}},
          pattern=LinePattern.Dot),
        Ellipse(
          extent={{-5,5},{5,-5}},
          pattern=LinePattern.None,
          lineColor={0,0,0},
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Text(
          extent={{3,-4},{33,-10}},
          lineColor={0,0,255},
          textString="states[2:n-1]"),
        Ellipse(
          extent={{95,5},{105,-5}},
          pattern=LinePattern.None,
          lineColor={0,0,0},
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Text(
          extent={{104,-4},{124,-10}},
          lineColor={0,0,255},
          textString="states[n]"),
        Ellipse(
          extent={{-105,5},{-95,-5}},
          pattern=LinePattern.None,
          lineColor={0,0,0},
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Text(
          extent={{-96,-4},{-76,-10}},
          lineColor={0,0,255},
          textString="states[1]"),
        Text(
          extent={{-99.5,30},{-69.5,24}},
          lineColor={0,0,255},
          textString="dimensions[1]"),
        Text(
          extent={{-0.5,30},{40,24}},
          lineColor={0,0,255},
          textString="dimensions[2:n-1]"),
        Text(
          extent={{100.5,30},{130.5,24}},
          lineColor={0,0,255},
          textString="dimensions[n]"),
        Line(
          points={{-34,73},{-34,52}},
          pattern=LinePattern.Dot),
        Line(
          points={{34,73},{34,57}},
          pattern=LinePattern.Dot),
        Line(
          points={{-100,50},{100,60}},
          thickness=0.5),
        Line(
          points={{-100,-50},{100,-60}},
          thickness=0.5),
        Line(
          points={{-100,73},{-100,50}},
          pattern=LinePattern.Dot),
        Line(
          points={{100,73},{100,60}},
          pattern=LinePattern.Dot),
        Line(
          points={{0,-55},{0,55}},
          arrow={Arrow.Filled,Arrow.Filled},
          pattern=LinePattern.Dot),
        Line(
          points={{-34,11},{34,11}},
          arrow={Arrow.None,Arrow.Filled}),
        Text(
          extent={{5,18},{25,12}},
          lineColor={0,0,255},
          textString="vs[2:n-1]"),
        Text(
          extent={{-72,18},{-62,12}},
          lineColor={0,0,255},
          textString="vs[1]"),
        Line(
          points={{-100,11},{-34,11}},
          arrow={Arrow.None,Arrow.Filled}),
        Text(
          extent={{63,18},{73,12}},
          lineColor={0,0,255},
          textString="vs[n]"),
        Line(
          points={{34,11},{100,11}},
          arrow={Arrow.None,Arrow.Filled}),
        Text(
          extent={{-80,-75},{-20,-81}},
          lineColor={0,0,255},
          textString="flowModel.pathLengths[1]"),
        Line(
          points={{-100,-82},{0,-82}},
          arrow={Arrow.Filled,Arrow.Filled}),
        Line(
          points={{0,-82},{100,-82}},
          arrow={Arrow.Filled,Arrow.Filled}),
        Text(
          extent={{15,-75},{85,-81}},
          lineColor={0,0,255},
          textString="flowModel.pathLengths[2:n-1]"),
        Text(
          extent={{-100,77},{-37,71}},
          lineColor={0,0,255},
          textString="lengths[1]"),
        Text(
          extent={{34,77},{100,71}},
          lineColor={0,0,255},
          textString="lengths[n]")}));
end PartialTwoPortPlugFlow2;
