within DCOL.Distribution.Pipes;
model DynamicPipePlugFlow "Dynamic pipe model with storage of mass and energy"

  import Modelica.Fluid.Types.ModelStructure;

  // extending PartialStraightPipe
  extends Modelica.Fluid.Pipes.BaseClasses.PartialStraightPipe(
    final port_a_exposesState = (modelStructure == ModelStructure.av_b) or (modelStructure == ModelStructure.av_vb),
    final port_b_exposesState = (modelStructure == ModelStructure.a_vb) or (modelStructure == ModelStructure.av_vb));

  // extending PartialTwoPortFlow
  extends BaseClasses.PartialTwoPortPlugFlow(
    final lengths=fill(length/n, n),
    final crossAreas=fill(crossArea, n),
    final dimensions=fill(4*crossArea/perimeter, n),
    final roughnesses=fill(roughness, n),
    final dheights=height_ab*dxs);

  // Wall heat transfer
  parameter Boolean use_HeatTransfer = false
    "= true to use the HeatTransfer model"
      annotation (Dialog(tab="Assumptions", group="Heat transfer"));
  replaceable model HeatTransfer =
      Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer
    constrainedby Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.PartialFlowHeatTransfer
    "Wall heat transfer"
      annotation (Dialog(tab="Assumptions", group="Heat transfer",enable=use_HeatTransfer),choicesAllMatching=true);
  Modelica.Fluid.Interfaces.HeatPorts_a[nNodes] heatPorts if use_HeatTransfer annotation (Placement(transformation(extent={{-10,45},{10,65}}), iconTransformation(extent={{-30,36},{32,52}})));

  HeatTransfer heatTransfer(
    redeclare final package Medium = Medium,
    final n=n,
    final nParallel=nParallel,
    final surfaceAreas=perimeter*lengths,
    final lengths=lengths,
    final dimensions=dimensions,
    final roughnesses=roughnesses,
    final states=mediums.state,
    final vs = vs,
    final use_k = use_HeatTransfer) "Heat transfer model"
      annotation (Placement(transformation(extent={{-45,20},{-23,42}})));
  final parameter Real[n] dxs = lengths/sum(lengths);
equation
  //Ts_ext = heatTransfer.heatPorts.T;
  //UA = heatTransfer.alphas.*(perimeter.*lengths);
  Qb_flows = heatTransfer.Q_flows;
  // Wb_flow = v*A*dpdx + v*F_fric
  //         = v*A*dpdx + v*A*flowModel.dp_fg - v*A*dp_grav
  if n == 1 or useLumpedPressure then
    Wb_flows = dxs * ((vs*dxs)*(crossAreas*dxs)*((port_b.p - port_a.p) + sum(flowModel.dps_fg) - system.g*(dheights*mediums.d)))*nParallel;
  else
    if modelStructure == ModelStructure.av_vb or modelStructure == ModelStructure.av_b then
      Wb_flows[2:n-1] = {vs[i]*crossAreas[i]*((mediums[i+1].p - mediums[i-1].p)/2 + (flowModel.dps_fg[i-1]+flowModel.dps_fg[i])/2 - system.g*dheights[i]*mediums[i].d) for i in 2:n-1}*nParallel;
    else
      Wb_flows[2:n-1] = {vs[i]*crossAreas[i]*((mediums[i+1].p - mediums[i-1].p)/2 + (flowModel.dps_fg[i]+flowModel.dps_fg[i+1])/2 - system.g*dheights[i]*mediums[i].d) for i in 2:n-1}*nParallel;
    end if;
    if modelStructure == ModelStructure.av_vb then
      Wb_flows[1] = vs[1]*crossAreas[1]*((mediums[2].p - mediums[1].p)/2 + flowModel.dps_fg[1]/2 - system.g*dheights[1]*mediums[1].d)*nParallel;
      Wb_flows[n] = vs[n]*crossAreas[n]*((mediums[n].p - mediums[n-1].p)/2 + flowModel.dps_fg[n-1]/2 - system.g*dheights[n]*mediums[n].d)*nParallel;
    elseif modelStructure == ModelStructure.av_b then
      Wb_flows[1] = vs[1]*crossAreas[1]*((mediums[2].p - mediums[1].p)/2 + flowModel.dps_fg[1]/2 - system.g*dheights[1]*mediums[1].d)*nParallel;
      Wb_flows[n] = vs[n]*crossAreas[n]*((port_b.p - mediums[n-1].p)/1.5 + flowModel.dps_fg[n-1]/2+flowModel.dps_fg[n] - system.g*dheights[n]*mediums[n].d)*nParallel;
    elseif modelStructure == ModelStructure.a_vb then
      Wb_flows[1] = vs[1]*crossAreas[1]*((mediums[2].p - port_a.p)/1.5 + flowModel.dps_fg[1]+flowModel.dps_fg[2]/2 - system.g*dheights[1]*mediums[1].d)*nParallel;
      Wb_flows[n] = vs[n]*crossAreas[n]*((mediums[n].p - mediums[n-1].p)/2 + flowModel.dps_fg[n]/2 - system.g*dheights[n]*mediums[n].d)*nParallel;
    elseif modelStructure == ModelStructure.a_v_b then
      Wb_flows[1] = vs[1]*crossAreas[1]*((mediums[2].p - port_a.p)/1.5 + flowModel.dps_fg[1]+flowModel.dps_fg[2]/2 - system.g*dheights[1]*mediums[1].d)*nParallel;
      Wb_flows[n] = vs[n]*crossAreas[n]*((port_b.p - mediums[n-1].p)/1.5 + flowModel.dps_fg[n]/2+flowModel.dps_fg[n+1] - system.g*dheights[n]*mediums[n].d)*nParallel;
    else
      assert(false, "Unknown model structure");
    end if;
  end if;

  connect(heatPorts, heatTransfer.heatPorts)
    annotation (Line(points={{0,55},{0,54},{-34,54},{-34,38.7}},
                                             color={191,0,0}));
  annotation (defaultComponentName="pipe",
Documentation(info="<html>
<p>Model of a straight pipe based on <a href=\"Modelica.Fluid.Pipes.DynamicPipe\">Modelica.Fluid.Pipes.DynamicPipe</a>. Only one modification was introduced to the original code. Instead of extending from <a href=\"Modelica.Fluid.Pipes.BaseClasses.PartialTwoPortFlow\">Modelica.Fluid.Pipes.BaseClasses.PartialTwoPortFlow</a>, the model extends from a plug flow based model (<a href=\"BaseClasses.PartialTwoPortPlugFlow\">BaseClasses.PartialTwoPortPlugFlow</a>).</p>
<p>More information about the plug flow model and its main assumptions is provided in the documentation of the class.</p>
</html>", revisions="<html>
<p>2016-05-31 <a href=\"mailto:jesus.febres@tekniker.es\">Jes&uacute;s Febres</a> (IK4-TEKNIKER): First implementation</p>
<p><b>Copyright &copy; <a href=\"http://www.tekniker.es/en\">IK4-TEKNIKER</a></b></p>
</html>"),
Icon(coordinateSystem(preserveAspectRatio=true,  extent={{-100,-100},{100,100}}), graphics={
        Rectangle(
          extent={{-100,44},{100,-44}},
          lineColor={0,0,0},
          fillPattern=FillPattern.HorizontalCylinder,
          fillColor={0,127,255}),
        Ellipse(
          extent={{-72,10},{-52,-10}},
          lineColor={0,0,0},
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Ellipse(
          extent={{50,10},{70,-10}},
          lineColor={0,0,0},
          fillColor={0,0,0},
          fillPattern=FillPattern.Solid),
        Text(
          extent={{-48,15},{46,-20}},
          lineColor={0,0,0},
          textString="%nNodes")}),
Diagram(coordinateSystem(preserveAspectRatio=true,  extent={{-100,-100},{100,
            100}}), graphics={
        Rectangle(
          extent={{-100,60},{100,50}},
          lineColor={0,0,0},
          fillColor={255,255,255},
          fillPattern=FillPattern.Backward),
        Rectangle(
          extent={{-100,-50},{100,-60}},
          lineColor={0,0,0},
          fillColor={255,255,255},
          fillPattern=FillPattern.Backward),
        Line(
          points={{100,45},{100,50}},
          arrow={Arrow.None,Arrow.Filled},
          pattern=LinePattern.Dot),
        Line(
          points={{0,45},{0,50}},
          arrow={Arrow.None,Arrow.Filled},
          pattern=LinePattern.Dot),
        Line(
          points={{100,-45},{100,-50}},
          arrow={Arrow.None,Arrow.Filled},
          pattern=LinePattern.Dot),
        Line(
          points={{0,-45},{0,-50}},
          arrow={Arrow.None,Arrow.Filled},
          pattern=LinePattern.Dot),
        Line(
          points={{-50,60},{-50,50}},
          pattern=LinePattern.Dot),
        Line(
          points={{50,60},{50,50}},
          pattern=LinePattern.Dot),
        Line(
          points={{0,-50},{0,-60}},
          pattern=LinePattern.Dot)}));
end DynamicPipePlugFlow;
