within DCOL.Generation.AbsorptionChillers.BaseClasses;
model AbsorberGeneratorVessel
  import SI = Modelica.SIunits;
  extends PartialLiquidVapourVessel(
    redeclare replaceable package Medium_v=Modelica.Media.Water.WaterIF97_R2pT,
    redeclare replaceable package Medium_l=LiBrH2O,
    final T_start=Tsat_start,
    final p_start=psat_start,
    T(stateSelect = StateSelect.prefer),
    V_l(stateSelect = StateSelect.prefer),
    medium_l(Xi(stateSelect = StateSelect.prefer)));

  parameter Boolean use_p_start = true
  "=true to use pressure during initialisation, otherwise temperature is used"
    annotation(Dialog(tab = "Initialization"));

  parameter SI.AbsolutePressure psat_start = if use_p_start then system.p_start
                                             else Medium_l.saturationPressure_TX(Tsat_start, X_l_start[1])
  "Initial liquid-vapor equilibrium pressure (saturation state)"
    annotation(Dialog(tab = "Initialization", enable=use_p_start));
  parameter SI.Temperature Tsat_start = if not use_p_start then system.T_start
                                        else Medium_l.temperature_psatX(psat_start, X_l_start[1])
  "Initial liquid-vapor equilibrium temperature (saturation state)"
    annotation(Dialog(tab = "Initialization",  enable=not use_p_start));

//Ports
  Modelica.Fluid.Interfaces.FluidPort_a port_v(redeclare package Medium = Medium_v)
  "Fluid port. Water vapor"
    annotation (Placement(transformation(extent={{-10,90},{10,110}}), iconTransformation(extent={{-10,90},{10,110}})));

  Modelica.Fluid.Interfaces.FluidPort_a port_l_a(redeclare package Medium = Medium_l)
  "Fluid port. LiBr to Generator"
    annotation (Placement(transformation(extent={{-70,-110},{-50,-90}}), iconTransformation(extent={{-70,-110},{-50,-90}})));

  Modelica.Fluid.Interfaces.FluidPort_b port_l_b(redeclare package Medium = Medium_l)
  "Fluid port. LiBr from Generator"
    annotation (Placement(transformation(extent={{48,-110},{68,-90}}), iconTransformation(extent={{48,-110},{68,-90}})));

  Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a heatPort
  "Heat port"
    annotation (Placement(transformation(extent={{-10,-10},{10,10}})));

equation
  Qb_flow = heatPort.Q_flow;
  heatPort.T = T;

  //Definition of port variables
  port_v.p = p;
  port_l_a.p = p;
  port_l_b.p = p_static;

  port_v.h_outflow = medium_v.h;
  port_l_a.h_outflow = medium_l.h;
  port_l_b.h_outflow = medium_l.h;

  port_v.Xi_outflow = medium_v.Xi;
  port_l_a.Xi_outflow = medium_l.Xi;
  port_l_b.Xi_outflow = medium_l.Xi;

  Hb_flow = port_v.m_flow*actualStream(port_v.h_outflow) + port_l_a.m_flow*actualStream(port_l_a.h_outflow) + port_l_b.m_flow*actualStream(port_l_b.h_outflow);
  Wb_flow = p * der(V_v);
  mb_flow = port_v.m_flow + port_l_a.m_flow + port_l_b.m_flow;
  mbXi_flow_v = port_v.m_flow*actualStream(port_v.Xi_outflow);
  mbXi_flow_l = port_l_a.m_flow*actualStream(port_l_a.Xi_outflow) + port_l_b.m_flow*actualStream(port_l_b.Xi_outflow);

  p = Medium_l.saturationPressure_TX(T, medium_l.Xi[1]);

  annotation (Icon(coordinateSystem(preserveAspectRatio=false)), Diagram(
        coordinateSystem(preserveAspectRatio=false)),
    Documentation(info="<html>
<p>Volume of constant size with three fluid ports and two medium models (liquid and vapour) at liquid-vapour equilibrium used to represent <b>the absorber or the generator of an absorption cycle</b>. The flow properties are computed from the upstream quantities, pressures are equal in all nodes. Heat transfer through a thermal port is possible, it equals zero if the port remains unconnected. Ideal heat transfer is assumed per default; the thermal port temperature is equal to the medium temperature. </p>
</html>", revisions="<html>
<p>2016-05-31 <a href=\"mailto:jesus.febres@tekniker.es\">Jes&uacute;s Febres</a> (IK4-TEKNIKER): First implementation</p>
<p><b>Copyright &copy; <a href=\"http://www.tekniker.es/en\">IK4-TEKNIKER</a></b></p>
</html>"));
end AbsorberGeneratorVessel;
