within DCOL.Generation.AbsorptionChillers.BaseClasses;
package LiBrH2O "LiBr + H2O"
  extends Modelica.Media.Interfaces.PartialMedium(
    mediumName="LiBrH2O",
    substanceNames={"LiBr", "H2O"},
    singleState=false,
    final ThermoStates=Modelica.Media.Interfaces.Choices.IndependentVariables.pTX);

  constant MolarMass MM_LiBr = 0.08685 "Molar mass of LiBr";
  constant MolarMass MM_H2O = 0.018015268 "Molar mass of water";

  redeclare record extends ThermodynamicState "Thermodynamic state"
    AbsolutePressure p "Pressure";
    Temperature T "Temperature";
    MassFraction[nX] X(start=reference_X) "Mass fractions (= (component mass)/total mass  m_i/m)";
  end ThermodynamicState;

  redeclare replaceable model extends BaseProperties(
    p(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default),
    T(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default),
    Xi(stateSelect = if preferredMediumStates then StateSelect.prefer else StateSelect.default))

  equation
    R = 0;
    MM = 1 / ((X[1]/MM_LiBr) + ((1-X[1])/MM_H2O));

    h = specificEnthalpy_TX(T, X[1]);
    d = density_TX(T, X[1]);

    u = h - p/d;

    state.p = p;
    state.T = T;
    state.X = X;
  end BaseProperties;

  redeclare function setState_pTX
    "Return thermodynamic state from p, T, and X or Xi"
    extends Modelica.Icons.Function;
    input AbsolutePressure p "Pressure";
    input Temperature T "Temperature";
    input MassFraction X[:] "Mass fractions";
    output ThermodynamicState state "Thermodynamic state record";
  algorithm
    state:=if size(X, 1) == nX then
      ThermodynamicState(p=p, T=T, X=X) else
      ThermodynamicState(p=p, T=T, X=cat(1, X, {1 - sum(X)}));
  end setState_pTX;

  redeclare function setState_phX
    "Return thermodynamic state from p, T, and X or Xi"
    extends Modelica.Icons.Function;
    input AbsolutePressure p "Pressure";
    input SpecificEnthalpy h "Specific enthazlpy of the solution";
    input MassFraction X[:] "Mass fractions";
    output ThermodynamicState state "Thermodynamic state record";
  algorithm
    state:=if size(X, 1) == nX then
      ThermodynamicState(p=p, T=temperature_hX(h, X[1]), X=X) else
      ThermodynamicState(p=p, T=temperature_hX(h, X[1]), X=cat(1, X, {1 - sum(X)}));
  end setState_phX;

  redeclare function extends pressure
  "Specific enthalpy w.r.t. thermodynamic state | use setState_phX function for input"
  algorithm
    p := state.p;
  end pressure;

  redeclare function extends temperature
  "Specific enthalpy w.r.t. thermodynamic state | use setState_phX function for input"
  algorithm
    T := state.T;
  end temperature;

  redeclare function extends specificEnthalpy
  "Specific enthalpy w.r.t. thermodynamic state | use setState_phX function for input"
  algorithm
    h := specificEnthalpy_TX(state.T, state.X[1]);
  end specificEnthalpy;

  redeclare function extends density
  "Specific enthalpy w.r.t. thermodynamic state | use setState_phX function for input"
  algorithm
    d := density_TX(state.T, state.X[1]);
  end density;

  redeclare function extends dynamicViscosity
  "Specific enthalpy w.r.t. thermodynamic state | use setState_phX function for input"
  algorithm
    eta := dynamicViscosity_TX(state.T, state.X[1]);
  end dynamicViscosity;

















  annotation (Icon(coordinateSystem(preserveAspectRatio=false)), Diagram(coordinateSystem(preserveAspectRatio=false)),
    Documentation(info="<html>
<p>Calculation of fluid properties for aqueous lithium-bromide (LiBr+H2O). The functions provided by this package shall be used inside of the restricted limits according to the referenced literature.</p>
<h4>References </h4>
<p><span style=\"font-family: MS Shell Dlg 2,sans-serif;\">Chua, H. T., H. K. Toh, A. Malek, K. C. Ng, and K. Srinivasan. 2000. &ldquo;Improved Thermodynamic Property Fields of LiBr-H2O Solution.&rdquo; <i>International Journal of Refrigeration</i> 23 (6): 412&ndash;29. doi:10.1016/S0140-7007(99)00076-6.</span> </p>
<p><span style=\"font-family: MS Shell Dlg 2,sans-serif;\">Consortium, Sorption Systems. n.d. &ldquo;LiBrSSC (Aqueous Lithium Bromide) Property Routines.&rdquo; Maryland. U.S.A.: University of Maryland. http://fchart.com/ees/libr_help/ssclibr.pdf.</span></p>
<p><span style=\"font-family: MS Shell Dlg 2,sans-serif;\">EDF. n.d. &ldquo;ThermoSysPro.&rdquo; Paris, France. https://build.openmodelica.org/Documentation/ThermoSysPro.html.</span> </p>
<p><span style=\"font-family: MS Shell Dlg 2,sans-serif;\">Kaita, Y. 2001. &ldquo;Thermodynamic Properties of Lithium Bromide - Water Solutions at High Temperatures.&rdquo; <i>International Journal of Refrigeration</i> 24 (5): 374&ndash;90. doi:10.1016/S0140-7007(00)00039-6.</span> </p>
<p><span style=\"font-family: MS Shell Dlg 2,sans-serif;\">McNeely, Lowell A. 1979. &ldquo;Thermodynamic Properties of Aqueous Solutions of Lithium Bromide.&rdquo; In <i>1979 Winter Conference</i>, 22. Philadelphia, PA: ASHRAE Transactions. http://www.techstreet.com/ashrae/standards/ph-79-03-3-thermodynamic-properties-of-aqueous-solutions-of-lithium-bromide?product_id=1853310.</span> </p>
</html>", revisions="<html>
<p>2016-05-31 <a href=\"mailto:jesus.febres@tekniker.es\">Jes&uacute;s Febres</a> (IK4-TEKNIKER): First implementation</p>
<p><b>Copyright &copy; <a href=\"http://www.tekniker.es/en\">IK4-TEKNIKER</a></b></p>
</html>"));
end LiBrH2O;
