within DCOL.Generation.AbsorptionChillers.BaseClasses;
partial model PartialLiquidVapourVessel "Vessel containing a vapor medium and a liquid medium at liquid-vapor equilibrium with mass and energy balance"
  import SI = Modelica.SIunits;
  import Modelica.Media.Interfaces.Choices.IndependentVariables;
  import Modelica.Fluid.Types.Dynamics;
  outer Modelica.Fluid.System system "System wide properties";

  replaceable package Medium_v = Modelica.Media.Water.WaterIF97_R2pT
    constrainedby Modelica.Media.Interfaces.PartialMedium
    "Vapor medium in vessel" annotation (choicesAllMatching=true);
  replaceable package Medium_l = Modelica.Media.Water.WaterIF97_R1pT
    constrainedby Modelica.Media.Interfaces.PartialMedium
    "Liquid medium in vessel" annotation (choicesAllMatching=true);

  // Tank geometry
  parameter SI.Height height "Height of vessel";
  parameter SI.Area crossArea "Area of vessel";
  final parameter SI.Volume V = height*crossArea "Volume of vessel";

  // Assumptions
  parameter Dynamics energyDynamics=Dynamics.FixedInitial
  "Formulation of energy balance"
    annotation(Evaluate=true, Dialog(tab = "Assumptions", group="Dynamics"));
  parameter Dynamics massDynamics=Dynamics.FixedInitial
  "Formulation of mass balance"
    annotation(Evaluate=true, Dialog(tab = "Assumptions", group="Dynamics"));
  final parameter Dynamics substanceDynamics=massDynamics
  "Formulation of mass balance"
    annotation(Evaluate=true, Dialog(tab = "Assumptions", group="Dynamics"));

  // Initialization
  parameter SI.Volume V_l_start(min=0, max=V) = 0.5*V
  "Initial volume of the liquid medium"
    annotation(Dialog(tab = "Initialization"));
  parameter SI.AbsolutePressure p_start = system.p_start
  "Initial liquid-vapor equilibrium pressure"
    annotation(Dialog(tab = "Initialization"));
  final parameter SI.AbsolutePressure p_static_start(fixed=false)
   "Static pressure at vessel bottom";
  parameter SI.Temperature T_start = system.T_start
  "Initial liquid-vapor equilibrium temperature"
    annotation(Dialog(tab = "Initialization"));
  parameter SI.MassFraction X_v_start[Medium_v.nX] = Medium_v.X_default
  "Initial X of vapor medium"
    annotation (Dialog(tab="Initialization", enable=Medium_l.nXi > 0));
  parameter SI.MassFraction X_l_start[Medium_l.nX] = Medium_l.X_default
  "Initial X of liquid medium"
    annotation (Dialog(tab="Initialization", enable=Medium_l.nXi > 0));

  SI.Volume V_l(min=0, max=V,
                start=V_l_start) "Liquid volume of the vessel";
  SI.Volume V_v(min=0, max=V,
                start=V-V_l_start) "Vapor volume of the vessel";
  SI.Height level "Level of liquid in the vessel";

  SI.Mass m "Mass in the vessel";
  SI.Mass m_l "Liquid mass of the vessel";
  SI.Mass m_v "Vapor mass in the vessel";
  SI.MassFlowRate mb_flow;

  SI.Mass[Medium_l.nXi] mXi_l;
  SI.MassFlowRate[Medium_l.nXi] mbXi_flow_l;
  SI.Mass[Medium_v.nXi] mXi_v;
  SI.MassFlowRate[Medium_v.nXi] mbXi_flow_v;

  SI.InternalEnergy U
    "Internal energy";
  SI.EnthalpyFlowRate Hb_flow
    "Heat flow across boundaries or energy source/sink";
  SI.HeatFlowRate Qb_flow
    "Heat flow across boundaries or energy source/sink";
  SI.Power Wb_flow
    "Work flow across boundaries or source term";

  Medium_v.BaseProperties medium_v(
    p(start=p_start),
    T(start=T_start),
    Xi(start=X_v_start[1:Medium_v.nXi], stateSelect = StateSelect.prefer),
    h(start=Medium_v.specificEnthalpy(Medium_v.setState_pTX(p_start, T_start, X_v_start))))
    "Vapor medium";
  Medium_l.BaseProperties medium_l(
    p(start=p_start),
    T(start=T_start),
    Xi(start=X_l_start[1:Medium_l.nXi], stateSelect = StateSelect.prefer),
    h(start=Medium_l.specificEnthalpy(Medium_l.setState_pTX(p_start, T_start, X_l_start))))
    "Liquid medium";

  SI.AbsolutePressure p(start=p_start)
    "Liquid-vapor equilibrium pressure in the vessel";
  SI.AbsolutePressure p_static
    "Static pressure at vessel bottom";
  SI.Temperature T(start=T_start)
    "Liquid-vapor equilibrium temperature in the vessel";

equation
  assert(V_l <= V, "Volume of liquid reaches the volume of the vessel");
  assert(V_l >= 0, "Volume of liquid reaches must be >=0");

  // Total quantities
  V_l = crossArea*level;
  V = V_l + V_v;
  m = m_l + m_v;
  m_l = V_l*medium_l.d;
  m_v = V_v*medium_v.d;
  mXi_l = m_l*medium_l.Xi;
  mXi_v = m_v*medium_v.Xi;
  U = m_l*medium_l.u + m_v*medium_v.u;

  // Liquid-vapor equilibrium
  p_static = level*Modelica.Constants.g_n*medium_l.d + p;
  medium_l.p = p;
  medium_v.p = p;
  medium_l.T = T;
  medium_v.T = T;

  // Energy and mass balances
  if energyDynamics == Dynamics.SteadyState then
    0 = Hb_flow + Qb_flow + Wb_flow;
  else
    der(U) = Hb_flow + Qb_flow + Wb_flow;
  end if;

  if massDynamics == Dynamics.SteadyState then
    0 = mb_flow;
    zeros(Medium_l.nXi) = mbXi_flow_l;
    zeros(Medium_v.nXi) = mbXi_flow_v;
  else
    der(m) = mb_flow;
    der(mXi_l) = mbXi_flow_l;
    der(mXi_v) = mbXi_flow_v;
  end if;

initial equation
  p_static_start = level*Modelica.Constants.g_n*medium_l.d + p_start;
  // initialization of balances
  if energyDynamics == Dynamics.FixedInitial then
    T = T_start;
  elseif energyDynamics == Dynamics.SteadyStateInitial then
    der(T) = 0;
  end if;

  if massDynamics == Dynamics.FixedInitial then
    V_l = V_l_start;
  elseif massDynamics == Dynamics.SteadyStateInitial then
    der(V_l) = 0;
  end if;

  if substanceDynamics == Dynamics.FixedInitial then
    medium_l.Xi = X_l_start[1:Medium_l.nXi];
    medium_v.Xi = X_v_start[1:Medium_v.nXi];
  elseif substanceDynamics == Dynamics.SteadyStateInitial then
    der(medium_l.Xi) = zeros(Medium_l.nXi);
    der(medium_v.Xi) = zeros(Medium_v.nXi);
  end if;

  annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
        Text(
          extent={{-100,160},{100,100}},
          lineColor={28,108,200},
          textString="%name"),
        Polygon(
          points={{-60,-100},{-100,-60},{-100,60},{-60,100},{60,100},{100,60},{100,-60},{60,-100},{-60,-100}},
          lineColor={28,108,200},
          fillColor={255,255,255},
          fillPattern=FillPattern.Solid),
        Polygon(
          points={{-28,36},{-28,36}},
          lineColor={28,108,200},
          fillColor={255,255,255},
          fillPattern=FillPattern.Solid),
        Polygon(
          points={{-100,-20},{100,-20},{100,-60},{60,-100},{-60,-100},{-100,-60},{-100,-20}},
          lineColor={28,108,200},
          fillColor={0,128,255},
          fillPattern=FillPattern.Solid)}),                      Diagram(
        coordinateSystem(preserveAspectRatio=false)),
    Documentation(info="<html>
<p>Base class for an ideal vessel with two mediums at liquid-vapor equilibrium with the ability to store mass and energy. The following boundary terms are part of the energy balance and must be specified in an extending class: </p>
<p><b>Qb_flow</b>, e.g., convective or latent heat flow rate across vessel boundary, and</p>
<p><b>Wb_flow</b>, work term.</p>
<p>Further source terms must be defined by an extending class for fluid flow across the segment boundary: </p>
<p><b>Hb_flow</b>, enthalpy flow,</p>
<p><b>mb_flow</b>, mass flow,</p>
<p><b>mbXi_flow_l</b> &AMP; <b>mbXi_flow_v</b>, substance mass flow for liquid and vapor medium respectively.</p>
<p>Since liquid-vapor equilibrium is assumped, temperatures (and pressures) of the vapor medium and the liquid medium are equal, and named <b>T</b> (<b>p</b>).</p>
</html>", revisions="<html>
<p>2016-05-31 <a href=\"mailto:jesus.febres@tekniker.es\">Jes&uacute;s Febres</a> (IK4-TEKNIKER): First implementation</p>
<p><b>Copyright &copy; <a href=\"http://www.tekniker.es/en\">IK4-TEKNIKER</a></b></p>
</html>"));
end PartialLiquidVapourVessel;
