within DCOL.Generation.ConventionalChillers.BaseClasses;
model RefrigerantCircuit_N "Refrigerant circuit model of a vapour-compression refrigeration system"
  extends Interfaces.PartialRefrigerantCircuit(
    control_in(unit="rev/min"));

//*********** Model parameters *************************************************//
  parameter Medium.Temperature T_low_nominal=0 +273.15
    "Evaporator temperature - refrigerant side (nominal)";
  parameter Medium.AbsolutePressure dp_eva_nominal=20
    "Evaporator pressure drop - refrigerant side (nominal)";
  parameter Modelica.SIunits.TemperatureDifference dT_superHeat_nominal=5
    "Super heating delta temperature (nominal)";

  parameter Medium.Temperature T_high_nominal=40 +273.15
    "Condenser temperature - refrigerant side (nominal)";
  parameter Medium.AbsolutePressure dp_con_nominal=20
    "Condenser pressure drop - refrigerant side (nominal)";
  parameter Modelica.SIunits.TemperatureDifference dT_subCool_nominal=5
    "Sub cooling delta temperature (nominal)";

  parameter Real valve_opening_nominal(
    min=0.0, max=1.0,
    final quantity="Percentage",
    final unit="1")=0.5 "Valve opening position (nominal)";

  parameter Modelica.SIunits.HeatFlowRate Q_flow_req_nominal=150e3
    "Evaporator heat flow rate (nominal)";
  parameter Modelica.SIunits.Power P_nominal=50e3
    "Compressor electrical power (nominal)";

  parameter Modelica.SIunits.Conversions.NonSIunits.AngularVelocity_rpm N_com_nominal=1500
    "Rotational speed, compressor";
  parameter Real consumption_coeff[3]={0.0, 1.0, 0.0}
    "Quadratic coeffitients of consumption calculation";

//*********** Model final parameters *******************************************//
  final parameter Medium.Temperature
    T_4_nominal=Medium.saturationTemperature(p_4_nominal);
  final parameter Medium.Temperature
    T_1_nominal=T_low_nominal+dT_superHeat_nominal;
  final parameter Medium.AbsolutePressure
    p_4_nominal=p_1_nominal+dp_eva_nominal;
  final parameter Medium.AbsolutePressure
    p_1_nominal=Medium.saturationPressure(T_low_nominal);
  final parameter Medium.SpecificEnthalpy
    h_4_nominal=h_3_nominal;
  final parameter Medium.SpecificEnthalpy
    h_1_nominal=Medium.specificEnthalpy(
      Medium.setState_pTX(p_1_nominal, T_1_nominal));

  final parameter Medium.Temperature
    T_2_nominal=T_high_nominal;
  final parameter Medium.Temperature
    T_3_nominal=T_high_nominal-dT_subCool_nominal;
  final parameter Medium.AbsolutePressure
    p_2_nominal=Medium.saturationPressure(T_2_nominal);
  final parameter Medium.AbsolutePressure
    p_3_nominal=p_2_nominal-dp_con_nominal;
  final parameter Medium.SpecificEnthalpy
    h_2_nominal=h_1_nominal + (P_nominal/m_flow_nominal);
  final parameter Medium.SpecificEnthalpy
    h_3_nominal=Medium.specificEnthalpy(Medium.setState_pTX(p_3_nominal, T_3_nominal));

  final parameter Modelica.SIunits.MassFlowRate
    m_flow_nominal=Q_flow_req_nominal/(h_1_nominal - h_4_nominal);
  final parameter Medium.AbsolutePressure
    dp_valve_nominal=p_3_nominal-p_4_nominal;

  final parameter Medium.Density
    rho_2_nominal=Medium.density_ph(p=p_2_nominal, h=h_2_nominal, phase=1);
  final parameter Real
    eta_com(min=0.0, max=1.0)=(p_2_nominal-p_1_nominal)*(m_flow_nominal/rho_2_nominal)/P_nominal;

  final parameter Medium.ThermodynamicState
    state1_nominal = Medium.setState_phX(p_1_nominal, h_1_nominal);
  final parameter Medium.ThermodynamicState
    state2_nominal = Medium.setState_phX(p_2_nominal, h_2_nominal);
  final parameter Medium.ThermodynamicState
    state3_nominal = Medium.setState_phX(p_3_nominal, h_3_nominal);
  final parameter Medium.ThermodynamicState
    state4_nominal = Medium.setState_phX(p_4_nominal, h_4_nominal);

//*********** Components ********************************************************//
  Modelica.Fluid.Pipes.DynamicPipe evaporator(
    redeclare package Medium = Medium,
    diameter=0.01,
    use_T_start=false,
    length=1,
    state_a(phase(start=1)),
    state_b(phase(start=1)),
    p_a_start=p_4_nominal,
    p_b_start=p_1_nominal,
    m_flow_start=m_flow_nominal,
    redeclare model FlowModel =
      FlowModels.NominalLaminarFlow_phX (
        m_flow_nominal=m_flow_nominal,
        dp_nominal(displayUnit="kPa")=dp_eva_nominal),
    nNodes=n,
    allowFlowReversal=false,
    h_start=h_1_nominal,
    nParallel=1,
    isCircular=true,
    roughness=2.5E-5,
    useLumpedPressure=false,
    modelStructure=Modelica.Fluid.Types.ModelStructure.av_vb,
    use_HeatTransfer=true)
    "Evaporator"
    annotation (Placement(transformation(
      extent={{-10,-52},{10,-72}},
      rotation=0)));

ControlledPump_N compressor(
  redeclare package Medium = Medium,
  final allowFlowReversal=false,
  final use_T_start=false,
  final V=0.01,
  final rho_nominal=rho_2_nominal,
  p_a_start=p_1_nominal,
  p_b_start=p_2_nominal,
  m_flow_start=m_flow_nominal,
  h_start=h_2_nominal,
  energyDynamics=Modelica.Fluid.Types.Dynamics.SteadyStateInitial,
  massDynamics=Modelica.Fluid.Types.Dynamics.SteadyStateInitial,
  checkValve=false,
  use_HeatTransfer=false,
  use_N_in=true,
  p_a_nominal=p_1_nominal,
  p_b_nominal=p_2_nominal,
  m_flow_nominal=m_flow_nominal,
  N_nominal=N_com_nominal,
    use_powerCharacteristic=false,
    consumption_coeff=consumption_coeff,
    redeclare function efficiencyCharacteristic =
        Modelica.Fluid.Machines.BaseClasses.PumpCharacteristics.constantEfficiency (                                          eta_nominal=eta_com))
  "Compressor"
  annotation (Placement(transformation(
      extent={{10,-10},{-10,10}},
      rotation=-90,
      origin={70,0})));

  Modelica.Fluid.Pipes.DynamicPipe condenser(
    redeclare package Medium = Medium,
    diameter=0.01,
    use_T_start=false,
    state_a(phase(start=1)),
    state_b(phase(start=1)),
    p_a_start=p_2_nominal,
    p_b_start=p_3_nominal,
    m_flow_start=m_flow_nominal,
    redeclare model FlowModel =
      FlowModels.NominalLaminarFlow_phX (
        m_flow_nominal=m_flow_nominal,
        dp_nominal(displayUnit="kPa")=dp_con_nominal),
    nNodes=n,
    allowFlowReversal=false,
    h_start=h_3_nominal,
    length=1,
    isCircular=true,
    nParallel=1,
    roughness=2.5E-5,
    useLumpedPressure=false,
    modelStructure=Modelica.Fluid.Types.ModelStructure.a_v_b,
    use_HeatTransfer=true)
    "Condenser"
    annotation (Placement(transformation(
      extent={{10,50},{-10,70}},
      rotation=0)));

  Modelica.Fluid.Vessels.ClosedVolume tank(
    nPorts=2,
    redeclare package Medium = Medium,
    use_portsData=false,
    p_start=p_3_nominal,
    use_T_start=false,
    h_start=h_3_nominal,
    massDynamics=Modelica.Fluid.Types.Dynamics.FixedInitial,
    V(displayUnit="l") = 0.005,
    use_HeatTransfer=false)
    annotation (Placement(transformation(
      extent={{-48,60},{-32,76}})));

  Modelica.Fluid.Valves.ValveLinear valve(
    redeclare package Medium = Medium,
    show_T=true,
    dp_start=dp_valve_nominal,
    m_flow_start=m_flow_nominal,
    allowFlowReversal=false,
    m_flow_nominal=m_flow_nominal,
    dp_nominal=valve_opening_nominal*dp_valve_nominal)
    "Thermal expansion valve"
    annotation (Placement(transformation(
      extent={{-10,-10},{10,10}},
      rotation=-90,
      origin={-70,0})));

  Modelica.Blocks.Continuous.LimPID valve_feedback(
    final yMax=1,
    final wp=1,
    final Ti=1,
    final controllerType=Modelica.Blocks.Types.SimpleController.PI,
    initType=Modelica.Blocks.Types.InitPID.NoInit,
    final xi_start=valve_feedback.yMin,
    final y_start=valve_opening_nominal,
    final k=valve_opening_nominal/dT_superHeat_nominal,
    final Ni=10,
    final yMin=0.1)
    "Valve P controller"
    annotation (Placement(transformation(extent={{-34,-6},{-46,6}})));
  Modelica.Fluid.Sensors.RelativeTemperature dT_superHeat(redeclare package Medium = Medium)
    annotation (Placement(transformation(extent={{-10,-26},{10,-46}})));

  Modelica.Blocks.Sources.Constant dT_superHeat_set(k=-dT_superHeat_nominal)
    annotation (Placement(transformation(extent={{6,-6},{-6,6}})));

    //*********** Outputs ***********************************************************//
  output Modelica.SIunits.MassFlowRate m_flow=valve.m_flow
    "Refrigerant mass flow rate in the valve";
  output Modelica.SIunits.HeatFlowRate Q_flow_total_eva = sum(evaporator.Qb_flows)
    "Evaporator heat flow rate";
  output Modelica.SIunits.HeatFlowRate Q_flow_total_con = sum(condenser.Qb_flows)
    "Condenser heat flow rate";
  // output Medium.ThermodynamicState state1=evaporator.state_b;
  output Medium.ThermodynamicState state1=Medium.setState_phX(compressor.port_a.p, inStream(compressor.port_a.h_outflow))
    "Refrigerant state at the evaporator output";
  // output Medium.ThermodynamicState state2=condenser.state_a;
  output Medium.ThermodynamicState state2=Medium.setState_phX(condenser.port_a.p, inStream(condenser.port_a.h_outflow))
    "Refrigerant state at the condenser input";
  // output Medium.ThermodynamicState state3=condenser.state_b;
  output Medium.ThermodynamicState state3=Medium.setState_phX(valve.port_a.p, inStream(valve.port_a.h_outflow))
    "Refrigerant state at the condenser output";
  // output Medium.ThermodynamicState state4=evaporator.state_a;
  output Medium.ThermodynamicState state4=Medium.setState_phX(evaporator.port_a.p, inStream(evaporator.port_a.h_outflow));

equation

  connect(compressor.port_b, condenser.port_a)
    annotation (Line(points={{70,10},{70,10},{70,60},{10,60}}, color={0,127,255}));
  connect(condenser.port_b, tank.ports[1])
    annotation (Line(points={{-10,60},{-41.6,60}},color={0,127,255}));
  connect(tank.ports[2], valve.port_a) annotation (Line(points={{-38.4,60},{-38.4,60},{-70,60},{-70,10}},
                              color={0,127,255}));
  connect(valve.port_b, evaporator.port_a)
    annotation (Line(points={{-70,-10},{-70,-10},{-70,-62},{-10,-62}}, color={0,127,255}));
  connect(evaporator.port_b, compressor.port_a)
    annotation (Line(points={{10,-62},{10,-62},{70,-62},{70,-10}}, color={0,127,255}));
  connect(condenser.heatPorts, heatPorts_b)
    annotation (Line(points={{-0.1,64.4},{-0.1,80.2},{0,80.2},{0,100}}, color={127,0,0}));
  connect(evaporator.heatPorts, heatPorts_a)
    annotation (Line(points={{0.1,-66.4},{0.1,-80.2},{0,-80.2},{0,-100}}, color={127,0,0}));
  connect(dT_superHeat.port_a, evaporator.port_a) annotation (Line(points={{-10,-36},{-20,-36},{-20,-62},{-10,-62}}, color={0,127,255}));
  connect(dT_superHeat.port_b, evaporator.port_b) annotation (Line(points={{10,-36},{20,-36},{20,-62},{10,-62}}, color={0,127,255}));
  connect(dT_superHeat.T_rel, valve_feedback.u_m) annotation (Line(points={{0,-27},{0,-20},{-40,-20},{-40,-7.2}}, color={0,0,127}));
  connect(dT_superHeat_set.y, valve_feedback.u_s) annotation (Line(points={{-6.6,0},{-32.8,0},{-32.8,0}}, color={0,0,127}));
  connect(valve_feedback.y, valve.opening) annotation (Line(points={{-46.6,0},{-62,0}}, color={0,0,127}));
  connect(control_in, compressor.N_in) annotation (Line(points={{120,0},{80,0}}, color={0,0,127}));
  annotation (                             Documentation(info="<html>
<p>Model of the refrigerant circuit included in a refrigeration system based on the classical vapour-compression cycle. As such systems, the model has four main components: a compressor, a condenser, a thermal expansion valve, and an evaporator. These component models are directly taken from Modelica Standard Library (MSL); however, they may be replaced by any model with the same model interface implementation.</p>
<p>In order to be set up, the model requires nominal values of an operation point, typically found in the manufacturer datasheets. These values are used to calculate the thermal conductance of both heat exchangers, the evaporator and the condenser (based on the modified <a href=\"Modelica_IK4_TEKNIKER.DCOL.Generation.ConventionalChillers.BaseClasses.DynamicPipe_inverseAlpha_ph\">DynamicPipe</a>). This estimation is carried out assuming that the heat is given (or taken) uniformly along the heat exchanger surface.</p>
<p>A PI controller model is included to emulate feedback loop of a thermostatic expansion valve and the evaporator output. In this version of the model, the controller parameters are not accessible and are set up so the super heating is kept at nominal value.</p>
<p>The <a href=\"Modelica_IK4_TEKNIKER.DCOL.Generation.ConventionalChillers.BaseClasses.PrescripPump_P\">compressor model</a> is based on the controlled pump model found in MSL. In particular, the pump power consumption can be used directly.</p>
<p>In addition, note that the refrigerant circuit includes a ClosedVolume model in order to avoid circular equalities.</p>
</html>", revisions="<html>
<p>2016-05-31 <a href=\"mailto:jesus.febres@tekniker.es\">Jes&uacute;s Febres</a> (IK4-TEKNIKER): First implementation</p>
<p><b>Copyright &copy; <a href=\"http://www.tekniker.es/en\">IK4-TEKNIKER</a></b></p>
</html>"), experiment(
      StopTime=120,
      Tolerance=0.001,
      __Dymola_Algorithm="Dassl"),
    __Dymola_Commands(file(ensureSimulated=true)=
        "modelica://Modelica/Resources/Scripts/Dymola/Fluid/HeatingSystem/plotResults.mos"
        "plotResults"),
    Diagram(coordinateSystem(preserveAspectRatio=false, extent={{-100,-100},{100,100}})),
    __Dymola_experimentSetupOutput,
    Icon(coordinateSystem(preserveAspectRatio=false, extent={{-100,-100},{100,100}})));
end RefrigerantCircuit_N;
