within DCOL.Generation.SolarThermal.BaseClasses;
model SimplifiedHeatTransferCollector_Reduced
  "Simplified description of heat transfer in a solar collector defined by EN 12975-2 standard, IncAng and I all together"

  parameter Real c1(unit="W/(m2.K)");
  parameter Real c2(unit="W/(m2.K2)");
  parameter Modelica.SIunits.HeatCapacity C;
  parameter Real F_tau_alfa(unit="1");
  parameter Modelica.SIunits.Area S_absorber "Solar collector absorber surface";

  Modelica.SIunits.HeatFlowRate HeatGain;
  Modelica.SIunits.HeatFlowRate HeatLosses;

  Modelica.SIunits.HeatFlowRate HeatStored;

  Modelica.SIunits.Temperature Tm "Average temperature of the collector";
  Real derTm;

  Modelica.SIunits.HeatFlowRate Q_flow "Heat flow rate from port_a -> port_b";
  Modelica.SIunits.TemperatureDifference dT "port_a.T - port_b.T";
public
  Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a port_a[3] annotation (
      Placement(transformation(extent={{-110,-10},{-90,10}}, rotation=0)));
  Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b port_b annotation (
      Placement(transformation(extent={{90,-10},{110,10}}, rotation=0)));

  Modelica.Blocks.Interfaces.RealInput iReaching annotation (Placement(
        transformation(
        extent={{-20,-20},{20,20}},
        rotation=90,
        origin={-56,-120})));

equation
  Tm = (port_a[3].T + port_a[1].T)/2;

  dT = Tm - port_b.T;
  port_a[2].Q_flow = Q_flow;
  port_a[1].Q_flow = 0;
  port_a[3].Q_flow = 0;
  port_b.Q_flow = -Q_flow;

  Q_flow = -(HeatGain - HeatLosses - HeatStored);

  derTm = der(Tm);
  HeatStored = C*derTm;
  HeatGain = S_absorber*F_tau_alfa*iReaching;
  HeatLosses = S_absorber*c1*dT + S_absorber*c2*dT^2;

  annotation (
    Diagram(coordinateSystem(preserveAspectRatio=false, extent={{-100,-100},{
            100,100}}), graphics),
    Icon(coordinateSystem(preserveAspectRatio=false, extent={{-100,-100},{100,
            100}}), graphics={Text(
          extent={{-58,130},{64,104}},
          lineColor={0,0,255},
          textString="%name",
          fontSize=40),Rectangle(extent={{-90,100},{90,-100}}, lineColor={0,0,0})}),
    Documentation(info="<html>
<p>This base class model is used to calculate the net heat flow transferred to the fluid in a solar collector. This calculation is carried out according to the corresponding expression at EN 12975 Test Standard:</p>
<p align=\"center\"><br/><img src=\"modelica://Ambassador_WP4/Resources/Images/Equation_SimplifiedModel_3.png\"/></p>
<p><br/>Model uses Irradiance and Incidence angle values (real inputs) to calculate the total input heat. There are also two heat interfaces: heatport_a and heatport_b.</p>
<p>Heatport_a is used to introuduce the ambient temperature value into the model (Ta).</p>
<p>Heatport_b, is a three element array heat port. Two of these are used to introduce temperature values equivalent to individual collector input and output, and thereby calculate the average temperature (Tm) supposed as linear distribution. With this temperature values (Tm, Ta) the heat losses are calculated. </p>
<p>Finally, the other heat port element is used to introduce the net heat flow into the fluid.</p>
</html>"));
end SimplifiedHeatTransferCollector_Reduced;
