within DCOL.Generation.SolarThermal;
model ParallelSimplifiedSolarCollector
  "Parallel arrangement of solar collectors based on simplified model"
  extends BaseClasses.SolarCollectorBasicInterface;

  parameter Integer Nparallel=3
    "Number of solar collectors in parallel arrangement";
  parameter Modelica.SIunits.MassFlowRate m_flow_max=0.041666
    "Maximum Mass flow rate of the collector"
    annotation (Dialog(tab="Collector Configuration and Sizing"));
  parameter Integer n_tubes=7
    "Number of tubes in the harp shape of the collector"
    annotation (Dialog(tab="Collector Configuration and Sizing"));
  parameter Modelica.SIunits.Length D_tubes=7*10^(-3)
    "Internal diameter of tubes in the harp shape of the collector"
    annotation (Dialog(tab="Collector Configuration and Sizing"));
  parameter Modelica.SIunits.Length L_tubes=1.94
    "Length of the tubes in the harp shape of the collector"
    annotation (Dialog(tab="Collector Configuration and Sizing"));
  parameter Modelica.SIunits.Area S_absorber=1.91 "Collector absorber surface"
    annotation (Dialog(tab="Collector Configuration and Sizing"));
  parameter Real incidenceAngleModifierData=0.93
    "Incidence angle modifier calculated for angleModifierData value"
    annotation (Dialog(tab="Collector Standardized Tests Data",group=
          "Incidence Angle Modifier"));
  parameter Modelica.SIunits.Angle angleModifierData=50*Modelica.Constants.pi/
      180 "Angle value at which incidence angle modifier has been calculated"
    annotation (Dialog(tab="Collector Standardized Tests Data",group=
          "Incidence Angle Modifier"));
  parameter Modelica.SIunits.HeatCapacity C=10228
    "Collector thermal capacity (Tested or Calculated)" annotation (Dialog(tab=
          "Collector Standardized Tests Data", group="Heat Capacity"));
  parameter Real F_tau_alfa=0.764
    "Collector optical efficiency (Constant therm)" annotation (Dialog(tab=
          "Collector Standardized Tests Data", group="Performance Curve"));
  parameter Real c1=3.539 "First order heat loss coefficient (Linear therm)"
    annotation (Dialog(tab="Collector Standardized Tests Data", group=
          "Performance Curve"));
  parameter Real c2=0.024
    "Second order heat loss coefficient (Quadratic therm)" annotation (Dialog(
        tab="Collector Standardized Tests Data", group="Performance Curve"));
  parameter Modelica.SIunits.VolumeFlowRate x[:]={6.166e-5,4.833e-5,3.5e-5,
      2.33e-5,1e-5}
    "Flow rate points measured for the pressure drop measurement" annotation (
      Dialog(tab="Collector Standardized Tests Data", group="Pressure Losses"));
  parameter Modelica.SIunits.Pressure y[:]={600,400,300,200,100}
    "Pressure drop measured at each flow rate value in x[:]" annotation (Dialog(
        tab="Collector Standardized Tests Data", group="Pressure Losses"));
  final parameter Real coeficients[2]=BaseClasses.LeastSquaresGenericResistance(
      x, y);
  final parameter Real a=coeficients[1]
    "Coefficient for quadratic term of a 2nd order polinomial function for relation between dp and V_flow of the collector";

  final parameter Real b=coeficients[2]
    "Coefficient for linear term of a 2nd order polinomial function for relation between dp and V_flow of the collector";
  Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature
    prescribedAmbientTemperature annotation (Placement(transformation(
        origin={-9,79},
        extent={{-5,-5},{5,5}},
        rotation=270)));
  Modelica.Fluid.Pipes.DynamicPipe          collectorHarp(
    use_HeatTransfer=true,
    redeclare package Medium = Medium,
    roughness=0,
    nNodes=3,
    nParallel=Nparallel*n_tubes,
    length=L_tubes,
    diameter=D_tubes,
    redeclare model FlowModel =
        Modelica.Fluid.Pipes.BaseClasses.FlowModels.NominalLaminarFlow (
          dp_nominal=0, m_flow_nominal=m_flow_max),
    modelStructure=Modelica.Fluid.Types.ModelStructure.av_vb,
    redeclare model HeatTransfer =
        Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer)
                                                   annotation (Placement(
        transformation(
        origin={-7,-74},
        extent={{9,10},{-9,-10}},
        rotation=180)));

  BaseClasses.SimplifiedHeatTransferCollector simplifiedHeatTransferCollector(
    c1=c1,
    c2=c2,
    F_tau_alfa=F_tau_alfa,
    S_absorber=Nparallel*S_absorber,
    incidenceAngleModifierData=incidenceAngleModifierData,
    angleModifierData=angleModifierData,
    C=Nparallel*C) annotation (Placement(transformation(
        extent={{-22,16},{22,-16}},
        rotation=90,
        origin={-8,22})));

  Modelica.Fluid.Fittings.GenericResistances.VolumeFlowRate
    pressureLoss(
    a=a/(Nparallel^2),
    b=b/Nparallel,
    redeclare package Medium = Medium)
    annotation (Placement(transformation(extent={{24,-84},{44,-64}})));

equation
  connect(inlet, collectorHarp.port_a) annotation (Line(
      points={{-110,-74},{-16,-74}},
      color={0,127,255},
      smooth=Smooth.None));
  connect(prescribedAmbientTemperature.port, simplifiedHeatTransferCollector.port_b)
    annotation (Line(
      points={{-9,74},{-8,74},{-8,44}},
      color={191,0,0},
      smooth=Smooth.None));
  connect(simplifiedHeatTransferCollector.port_a, collectorHarp.heatPorts)
    annotation (Line(
      points={{-8,0},{-6.91,0},{-6.91,-69.6}},
      color={191,0,0},
      smooth=Smooth.None));
  connect(collectorHarp.port_b, pressureLoss.port_a) annotation (Line(
      points={{2,-74},{24,-74}},
      color={0,127,255},
      smooth=Smooth.None));
  connect(pressureLoss.port_b, outlet) annotation (Line(
      points={{44,-74},{48,-74},{48,-77},{92,-77}},
      color={0,127,255},
      smooth=Smooth.None));
  connect(solarCollectorInput.AmbientTemperature, prescribedAmbientTemperature.T)
    annotation (Line(
      points={{-1.915,133.09},{-6,133.09},{-6,85},{-9,85}},
      color={0,0,127},
      smooth=Smooth.None), Text(
      string="%first",
      index=-1,
      extent={{-6,3},{-6,3}}));
  connect(simplifiedHeatTransferCollector.irradiance, solarCollectorInput.Irradiance)
    annotation (Line(
      points={{-27.2,33},{-62,33},{-62,133.09},{-1.915,133.09}},
      color={0,0,127},
      smooth=Smooth.None), Text(
      string="%second",
      index=1,
      extent={{6,3},{6,3}}));
  connect(simplifiedHeatTransferCollector.incidenceAngle, solarCollectorInput.IncidenceAngle)
    annotation (Line(
      points={{-27.2,9.68},{-92,9.68},{-92,133.09},{-1.915,133.09}},
      color={0,0,127},
      smooth=Smooth.None), Text(
      string="%second",
      index=1,
      extent={{6,3},{6,3}}));
  annotation (
    Diagram(coordinateSystem(preserveAspectRatio=false, extent={{-140,-120},{
            140,140}}), graphics),
    Documentation(info="<html>
<p><h4><font color=\"#008000\">Model description</font></h4></p>
<p>Based on the simplified model, the parallel connection of N solar collectors is equivalent to a unique solar collector with an area N times bigger, an N times higher heat capacity, an N times higher number of tubes, and the same pressure drop. Thus, just one simplifiedHeatTransferCollector is used, just modifying its parameters.</p>
<p>In the developed model (ParllelArray model) the user just has to set up the number of solar collectors in the collector array. </p>
<p><b><font style=\"color: #008000; \">Model interfaces</font></b> </p>
<p>In case this model is integrated to simulate the behaviour of the array when exposed to the solar radiation with a fluid circulating inside, the necessary inputs and available outputs would be the following: </p>
<p><i>Inputs:</i> </p>
<p><ul>
<li>The irradiance in the plane of the collector (W/m2) </li>
<li>The incidence angle (rad) </li>
<li>The ambient temperature (K) </li>
<li>Fluid input port conditions (i.e. flow rate and temperature)</li>
</ul></p>
<p>The first three inputs should be RealInput Modelica type. </p>
<p><i>Outputs:</i> </p>
<p>Due to the capacities of Modelica language, the value of all the variables used by the solver for the model solution is accessible after simulation. Here just the most interesting ones are mentioned </p>
<p><ul>
<li>Fluid temperatures: inlet, outlet, and the arithmetic mean. </li>
<li>Disaggregated heat flows: input, losses, fluid, and storage. </li>
</ul></p>
<p>------------------------------------------------------------------------------------------------------------------------</p>
</html>"),
    Icon(coordinateSystem(preserveAspectRatio=false, extent={{-140,-120},{140,
            140}}), graphics));
end ParallelSimplifiedSolarCollector;
