within DCOL.Generation.SolarThermal;
model SerieSimplifiedSolarCollector "Solar collector Serie arrangement"
  extends BaseClasses.SolarCollectorBasicInterface;

  parameter Integer Nseries=3 "Number of solar collectors in serie arrangement";
  parameter Real Ptg=0
    "Increment (%) in the module Pressure Drop due to additional pipe length";

  parameter Modelica.SIunits.MassFlowRate m_flow_max=0.041666
    "Maximum Mass flow rate of the collector"
    annotation (Dialog(tab="Configuration and Sizing"));
  parameter Integer n_tubes=7
    "Number of tubes in the harp shape of the collector"
    annotation (Dialog(tab="Configuration and Sizing"));
  parameter Modelica.SIunits.Length D_tubes=7*10^(-3)
    "Internal diameter of tubes in the harp shape of the collector"
    annotation (Dialog(tab="Configuration and Sizing"));
  parameter Modelica.SIunits.Length L_tubes=1.94
    "Length of the tubes in the harp shape of the collector"
    annotation (Dialog(tab="Configuration and Sizing"));
  parameter Modelica.SIunits.Area S_absorber=1.91 "Collector absorber surface"
    annotation (Dialog(tab="Configuration and Sizing"));
  parameter Real incidenceAngleModifierData=0.93
    "Incidence angle modifier calculated for angleModifierData value"
    annotation (Dialog(tab="Standardized Tests Data",group=
          "Incidence Angle Modifier"));
  parameter Modelica.SIunits.Angle angleModifierData=50*Modelica.Constants.pi/
      180 "Angle value at which incidence angle modifier has been calculated"
    annotation (Dialog(tab="Standardized Tests Data",group=
          "Incidence Angle Modifier"));
  parameter Modelica.SIunits.HeatCapacity C=10228
    "Collector thermal capacity (Tested or Calculated)"
    annotation (Dialog(tab="Standardized Tests Data", group="Heat Capacity"));
  parameter Real F_tau_alfa=0.764
    "Collector optical efficiency (Constant therm)" annotation (Dialog(tab=
          "Standardized Tests Data", group="Performance Curve"));
  parameter Real c1=3.539 "First order heat loss coefficient (Linear therm)"
    annotation (Dialog(tab="Standardized Tests Data", group="Performance Curve"));
  parameter Real c2=0.024
    "Second order heat loss coefficient (Quadratic therm)" annotation (Dialog(
        tab="Standardized Tests Data", group="Performance Curve"));
  parameter Modelica.SIunits.VolumeFlowRate x[:]={6.166e-5,4.833e-5,3.5e-5,
      2.33e-5,1e-5}
    "Flow rate points measured for the pressure drop measurement"
    annotation (Dialog(tab="Standardized Tests Data",group="Pressure Losses"));
  parameter Modelica.SIunits.Pressure y[:]={600,400,300,200,100}
    "Pressure drop measured at each flow rate value in x[:]"
    annotation (Dialog(tab="Standardized Tests Data",group="Pressure Losses"));
  final parameter Real coeficients[2]=BaseClasses.LeastSquaresGenericResistance(
      x, Nseries*(1 + Ptg/100)*y);
  final parameter Real a=coeficients[1]
    "Coefficient for quadratic term of a 2nd order polinomial function for relation between dp and V_flow of the collector";

  final parameter Real b=coeficients[2]
    "Coefficient for linear term of a 2nd order polinomial function for relation between dp and V_flow of the collector";

  final parameter Integer nNodes=Nseries+2
    "Number of discrete flow volumes/heat ports";

  Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature
    prescribedAmbientTemperature annotation (Placement(transformation(
        origin={-9,79},
        extent={{-5,-5},{5,5}},
        rotation=270)));
  Modelica.Fluid.Pipes.DynamicPipe          collectorHarp(
    use_HeatTransfer=true,
    redeclare package Medium = Medium,
    roughness=0,
    nParallel=n_tubes,
    length=Nseries*L_tubes,
    diameter=D_tubes,
    nNodes=nNodes,
    redeclare model FlowModel =
        Modelica.Fluid.Pipes.BaseClasses.FlowModels.NominalLaminarFlow (
          dp_nominal=0, m_flow_nominal=m_flow_max),
    modelStructure=Modelica.Fluid.Types.ModelStructure.av_vb,
    redeclare model HeatTransfer =
        Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer)
                   annotation (Placement(transformation(
        origin={-8,-74},
        extent={{12,12},{-12,-12}},
        rotation=180)));

  BaseClasses.SimplifiedHeatTransferCollector simplifiedHeatTransferCollector[
    Nseries](
    c1=c1,
    c2=c2,
    F_tau_alfa=F_tau_alfa,
    S_absorber=S_absorber,
    incidenceAngleModifierData=incidenceAngleModifierData,
    angleModifierData=angleModifierData,
    C=C) annotation (Placement(transformation(
        extent={{-22,16},{22,-16}},
        rotation=90,
        origin={-8,22})));

  Modelica.Fluid.Fittings.GenericResistances.VolumeFlowRate
    pressureLoss(
    a=a,
    b=b,
    redeclare package Medium = Medium)
    annotation (Placement(transformation(extent={{24,-84},{44,-64}})));

equation
  for i in 1:Nseries loop

    connect(simplifiedHeatTransferCollector[i].port_a[1], collectorHarp.heatPorts[
      i]) annotation (Line(
        points={{-9.06667,0},{-9.06667,-68.72},{-7.88,-68.72}},
        color={191,0,0},
        smooth=Smooth.None));
    connect(simplifiedHeatTransferCollector[i].port_a[2], collectorHarp.heatPorts[
      i + 1]) annotation (Line(
        points={{-8,0},{-8,-68.72},{-7.88,-68.72}},
        color={191,0,0},
        smooth=Smooth.None));
    connect(simplifiedHeatTransferCollector[i].port_a[3], collectorHarp.heatPorts[
      i + 1]) annotation (Line(
        points={{-6.93333,0},{-6.93333,-68.72},{-7.88,-68.72}},
        color={191,0,0},
        smooth=Smooth.None));
    connect(simplifiedHeatTransferCollector[i].port_b,
      prescribedAmbientTemperature.port) annotation (Line(
        points={{-8,44},{-8,60},{-8,74},{-9,74}},
        color={191,0,0},
        smooth=Smooth.None));
    connect(solarCollectorInput.Irradiance, simplifiedHeatTransferCollector[i].irradiance)
      annotation (Line(
        points={{-1.915,133.09},{-98,133.09},{-98,33},{-27.2,33}},
        color={0,0,0},
        smooth=Smooth.None), Text(
        string="%first",
        index=-1,
        extent={{-6,3},{-6,3}}));
    connect(solarCollectorInput.IncidenceAngle, simplifiedHeatTransferCollector[
      i].incidenceAngle) annotation (Line(
        points={{-1.915,133.09},{-122,133.09},{-122,9.68},{-27.2,9.68}},
        color={0,0,0},
        smooth=Smooth.None), Text(
        string="%first",
        index=-1,
        extent={{-6,3},{-6,3}}));

  end for;
  connect(inlet, collectorHarp.port_a) annotation (Line(
      points={{-110,-74},{-20,-74}},
      color={0,127,255},
      smooth=Smooth.None));
  connect(collectorHarp.port_b, pressureLoss.port_a) annotation (Line(
      points={{4,-74},{24,-74}},
      color={0,127,255},
      smooth=Smooth.None));
  connect(pressureLoss.port_b, outlet) annotation (Line(
      points={{44,-74},{66,-74},{66,-77},{92,-77}},
      color={0,127,255},
      smooth=Smooth.None));

  connect(solarCollectorInput.AmbientTemperature, prescribedAmbientTemperature.T)
    annotation (Line(
      points={{-1.915,133.09},{-6,133.09},{-6,85},{-9,85}},
      color={0,0,127},
      smooth=Smooth.None), Text(
      string="%first",
      index=-1,
      extent={{-6,3},{-6,3}}));
  annotation (
    Line(
      points={{-8,0},{-7.88,0},{-7.88,-68.72}},
      color={191,0,0},
      smooth=Smooth.None),
    Diagram(coordinateSystem(preserveAspectRatio=false, extent={{-140,-120},{
            140,140}}), graphics),
    Documentation(info="<html>
<p><h4><font color=\"#008000\">Model description</font></h4></p>
<p>The series connection of N solar collectors is not so easy to simplify as the parallel one, because the mean fluid temperature (Tm), which is needed for the heat loss and heat storage calculation, is not the same at every collector and cannot be easily extrapolated.</p>
<p>In this case the developed model simulates a fluid line N times longer than at a single collector (with the same number of tubes), discretized into N+2 nodes. The net heat flow calculated at each solar collector (<a href=\"Ambassador_WP4.ThermoFluidTek.ThermalSources.SolarThermal.BaseClasses.SimplifiedHeatTransferCollector\">SimplifiedHeatTransferCollector</a>) is introduced at each one of the N central nodes. </p>
<p>In the developed model (SeriesArray model), apart from simplified collector model parameters, the user has to set up the number of solar collectors in the collector array, and the &percnt; of additional pressure drop (Pctg) with respect to the theoretical array pressure drop (N times individual solar collector pressure drop) due to additional pipe length for the connections. </p>
<p><b><font style=\"color: #008000; \">Model interfaces</font></b> </p>
<p>In case this model is integrated to simulate the behaviour of the collector when exposed to the solar radiation with a fluid circulating inside, the necessary inputs and available outputs would be the following: </p>
<p><i>Inputs:</i> </p>
<p><ul>
<li>The irradiance in the plane of the collector (W/m2) </li>
<li>The incidence angle (rad) </li>
<li>The ambient temperature (K) </li>
<li>Fluid input port conditions (i.e. flow rate and temperature)</li>
</ul></p>
<p>The first three inputs should be RealInput Modelica type. </p>
<p><i>Outputs:</i> </p>
<p>Due to the capacities of Modelica language, the value of all the variables used by the solver for the model solution is accessible after simulation. Here just the most interesting ones are mentioned </p>
<p><ul>
<li>Fluid temperatures: inlet, outlet, and the arithmetic mean. </li>
<li>Disaggregated heat flows: input, losses, fluid, and storage. </li>
</ul></p>
<p>------------------------------------------------------------------------------------------------------------------------</p>
</html>"),
    Icon(coordinateSystem(preserveAspectRatio=false, extent={{-140,-120},{140,
            140}}), graphics));
end SerieSimplifiedSolarCollector;
