within DCOL.Generation.SolarThermal;
model SimplifiedSolarCollector "Simple model for a solar collector"
  extends BaseClasses.SolarCollectorBasicInterface;

  parameter Modelica.SIunits.MassFlowRate m_flow_max=0.041666
    "Maximum Mass flow rate of the collector"
    annotation (Dialog(tab="Configuration and Sizing"));
  parameter Integer n_tubes=7
    "Number of tubes in the harp shape of the collector"
    annotation (Dialog(tab="Configuration and Sizing"));
  parameter Modelica.SIunits.Length D_tubes=7*10^(-3)
    "Internal diameter of tubes in the harp shape of the collector"
    annotation (Dialog(tab="Configuration and Sizing"));
  parameter Modelica.SIunits.Length L_tubes=1.94
    "Length of the tubes in the harp shape of the collector"
    annotation (Dialog(tab="Configuration and Sizing"));
  parameter Modelica.SIunits.Area S_absorber=1.91 "Collector absorber surface"
    annotation (Dialog(tab="Configuration and Sizing"));
  parameter Real incidenceAngleModifierData=0.93
    "Incidence angle modifier calculated for angleModifierData value"
    annotation (Dialog(tab="Standardized Tests Data",group=
          "Incidence Angle Modifier"));
  parameter Modelica.SIunits.Angle angleModifierData=50*Modelica.Constants.pi/
      180 "Angle value at which incidence angle modifier has been calculated"
    annotation (Dialog(tab="Standardized Tests Data",group=
          "Incidence Angle Modifier"));
  parameter Modelica.SIunits.HeatCapacity C=10228
    "Collector thermal capacity (Tested or Calculated)"
    annotation (Dialog(tab="Standardized Tests Data", group="Heat Capacity"));
  parameter Real F_tau_alfa=0.764
    "Collector optical efficiency (Constant therm)" annotation (Dialog(tab=
          "Standardized Tests Data", group="Performance Curve"));
  parameter Real c1=3.539 "First order heat loss coefficient (Linear therm)"
    annotation (Dialog(tab="Standardized Tests Data", group="Performance Curve"));
  parameter Real c2=0.024
    "Second order heat loss coefficient (Quadratic therm)" annotation (Dialog(
        tab="Standardized Tests Data", group="Performance Curve"));
  parameter Modelica.SIunits.VolumeFlowRate x[:]={6.166e-5,4.833e-5,3.5e-5,
      2.33e-5,1e-5}
    "Flow rate points measured for the pressure drop measurement"
    annotation (Dialog(tab="Standardized Tests Data",group="Pressure Losses"));
  parameter Modelica.SIunits.Pressure y[:]={600,400,300,200,100}
    "Pressure drop measured at each flow rate value in x[:]"
    annotation (Dialog(tab="Standardized Tests Data",group="Pressure Losses"));
  final parameter Real coeficients[2]=BaseClasses.LeastSquaresGenericResistance(
      x, y);
  final parameter Real a=coeficients[1]
    "Coefficient for quadratic term of a 2nd order polinomial function for relation between dp and V_flow of the collector";

  final parameter Real b=coeficients[2]
    "Coefficient for linear term of a 2nd order polinomial function for relation between dp and V_flow of the collector";

  Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature
    prescribedAmbientTemperature annotation (Placement(transformation(
        origin={-1,79},
        extent={{-5,-5},{5,5}},
        rotation=270)));
  Modelica.Fluid.Pipes.DynamicPipe          collectorHarp(
    use_HeatTransfer=true,
    redeclare package Medium = Medium,
    roughness=0,
    nNodes=3,
    nParallel=n_tubes,
    length=L_tubes,
    diameter=D_tubes,
    redeclare model HeatTransfer =
        Modelica.Fluid.Pipes.BaseClasses.HeatTransfer.IdealFlowHeatTransfer,
    modelStructure=Modelica.Fluid.Types.ModelStructure.av_vb,
    redeclare model FlowModel =
        Modelica.Fluid.Pipes.BaseClasses.FlowModels.NominalLaminarFlow (
          dp_nominal=0, m_flow_nominal=m_flow_max))
                                                   annotation (Placement(
        transformation(
        origin={-1,-74},
        extent={{9,10},{-9,-10}},
        rotation=180)));

  BaseClasses.SimplifiedHeatTransferCollector simplifiedHeatTransferCollector(
    c1=c1,
    c2=c2,
    F_tau_alfa=F_tau_alfa,
    S_absorber=S_absorber,
    incidenceAngleModifierData=incidenceAngleModifierData,
    angleModifierData=angleModifierData,
    C=C) annotation (Placement(transformation(
        extent={{-22,16},{22,-16}},
        rotation=90,
        origin={0,22})));

  Modelica.Fluid.Fittings.GenericResistances.VolumeFlowRate
    pressureLoss(
    a=a,
    b=b,
    redeclare package Medium = Medium)
    annotation (Placement(transformation(extent={{32,-84},{52,-64}})));

equation
  connect(inlet, collectorHarp.port_a) annotation (Line(
      points={{-110,-74},{-10,-74}},
      color={0,127,255},
      smooth=Smooth.None));
  connect(prescribedAmbientTemperature.port, simplifiedHeatTransferCollector.port_b)
    annotation (Line(
      points={{-1,74},{0,74},{0,44}},
      color={191,0,0},
      smooth=Smooth.None));
  connect(simplifiedHeatTransferCollector.port_a, collectorHarp.heatPorts)
    annotation (Line(
      points={{0,0},{-0.91,0},{-0.91,-69.6}},
      color={191,0,0},
      smooth=Smooth.None));
  connect(collectorHarp.port_b, pressureLoss.port_a) annotation (Line(
      points={{8,-74},{32,-74}},
      color={0,127,255},
      smooth=Smooth.None));
  connect(pressureLoss.port_b, outlet) annotation (Line(
      points={{52,-74},{56,-74},{56,-77},{92,-77}},
      color={0,127,255},
      smooth=Smooth.None));
  connect(prescribedAmbientTemperature.T, solarCollectorInput.AmbientTemperature)
    annotation (Line(
      points={{-1,85},{-1,107.5},{-1.915,107.5},{-1.915,133.09}},
      color={0,0,127},
      smooth=Smooth.None), Text(
      string="%second",
      index=1,
      extent={{6,3},{6,3}}));
  connect(simplifiedHeatTransferCollector.irradiance, solarCollectorInput.Irradiance)
    annotation (Line(
      points={{-19.2,33},{-62,33},{-62,128},{-1.915,128},{-1.915,133.09}},
      color={0,0,127},
      smooth=Smooth.None), Text(
      string="%second",
      index=1,
      extent={{6,3},{6,3}}));
  connect(simplifiedHeatTransferCollector.incidenceAngle, solarCollectorInput.IncidenceAngle)
    annotation (Line(
      points={{-19.2,9.68},{-92,9.68},{-92,126},{-1.915,126},{-1.915,133.09}},
      color={0,0,127},
      smooth=Smooth.None), Text(
      string="%second",
      index=1,
      extent={{6,3},{6,3}}));
  annotation (
    Diagram(coordinateSystem(preserveAspectRatio=false, extent={{-140,-120},{
            140,140}})),
    Documentation(info="<html>
<p><h4><font color=\"#008000\">Model description</font></h4></p>
<p>This collector model is developed according to collector characterization parameters obtained from the EN12975 standardized steady state tests. This standard establishes the following collector performance curve:</p>
<p align=\"center\"><img src=\"modelica://Ambassador_WP4/Resources/Images/Equation_SimplifiedModel_1.png\"/></p>
<p>where, </p>
<p><img src=\"modelica://Ambassador_WP4/Resources/equations/equation-eYT0AViz.png\" alt=\"K_theta\"/><img src=\"modelica://Ambassador_WP4/Resources/equations/equation-mM9EzURp.png\" alt=\"(theta)\"/> Incidence angle modifier curve, represented by the following equation </p>
<p align=\"center\"><br/><img src=\"modelica://Ambassador_WP4/Resources/Images/Equation_SimplifiedModel_2.png\"/></p>
<p>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;where b0 is calculated from the value of <img src=\"modelica://Ambassador_WP4/Resources/equations/equation-eYT0AViz.png\" alt=\"K_theta\"/>(50&ordm;) given by standardized tests </p>
<p><br/><img src=\"modelica://Ambassador_WP4/Resources/equations/equation-T2HBa7kD.png\" alt=\"eta_0\"/>, <img src=\"modelica://Ambassador_WP4/Resources/equations/equation-PC1a3k5Z.png\" alt=\"a_1\"/>, <img src=\"modelica://Ambassador_WP4/Resources/equations/equation-jZfTtO3B.png\" alt=\"a_2\"/>, <img src=\"modelica://Ambassador_WP4/Resources/equations/equation-ztjCdz1c.png\" alt=\"mC_e\"/> Parameters obtained from the thermal performance standardized tests. </p>
<p>&nbsp; </p>
<p>For the modelization, the collector performance curve is expressed in the following terms: </p>
<p align=\"center\"><img src=\"modelica://Ambassador_WP4/Resources/Images/Equation_SimplifiedModel_3.png\"/></p>
<p><br/>In this model, only the fluid flow is modelled with an already developed model from the Modelica Standard Library, the Dynamic Pipe model, explained at section 3.2.1. But in this case, this standard model is modified in the following way: </p>
<p><ul>
<li>Pressure losses are nullified. The measurement of the collector pressure loss, although optional, is included in the mentioned standard tests. So an ad-hoc component has been developed and included in the model apart from the Dynamic Pipe in order to apply the corresponding total pressure losses according to the data obtained from the tests (see below). </li>
<li>Ideal heat transfer.<i> </i>No thermal resistance is applied between the fluid and the surrounding element. Actually in this case the surrounding element does not exist as the mass of the collector is lumped. However, it was proceed in this way in order to get the fluid temperature for energy losses and energy storage calculations. </li>
</ul></p>
<p>As at the detailed model, the Dynamic Pipe model includes a drop-down selector letting the user choose the appropriate fluid. </p>
<p>Apart from the dynamic pipe, another two components have been developed and included to complete the model: volumeFlowRate and simplifiedHeatTransferCollector. </p>
<p>As mentioned before, related to hydraulic behaviour a new component has been developed in order to calculate the pressure loss from the data obtained from the standardized tests (&lsquo;<u>volumeFlowRate&rsquo;</u>). This component initially applies least square method to approximate the test data to a 2nd order polynomial function without independent term, for later calculation of pressure drop depending on flow rate during simulation. </p>
<p>With respect to the thermal behaviour, another component has been included in order to calculate the right side of eq.13 (<u>simplifiedHeatTransferCollector</u>). This component thus includes: </p>
<p><ul>
<li><i>Total input energy</i>, from standardized test parameters, collector parameters and model variables (includes the incidence angle modifier calculation) </li>
<li><i>Energy losses</i>, according to parameters obtained from the standardized tests, collector parameters, the mean temperature value and other model variables. </li>
<li><i>Energy storage</i>, from standardized tests parameters and the variation of mean fluid temperature. </li>
</ul></p>
<p>It must be pointed out that <i>Tm</i> in the equations is the arithmetic mean between the inlet and outlet temperatures of the collector, thus supposing a linear temperature distribution in the collector, as some authors did before. However, according to other authors and to simulations shown later on, in reality this is not so. On the other hand, at the EN 12975 standardized test the arithmetic mean is used for the calculation of the collector thermal performance parameters, so as this standard is the reference for the simplified model, the arithmetic mean will be also used in this model according to previous equations. </p>
<p>As the fluid temperature distribution in the Dynamic Pipe is not linear, the arithmetic mean has to be calculated at &lsquo;simplifiedHeatTransferCollector&rsquo; component, coming from the input and output fluid temperatures at the Dynamic Pipe. </p>
<p><b></font><font style=\"color: #008000; \">Model interfaces</font></b> </p>
<p>In case this model is integrated to simulate the behaviour of the collector when exposed to the solar radiation with a fluid circulating inside, the necessary inputs and available outputs would be the following: </p>
<p><i>Inputs:</i> </p>
<p><ul>
<li>The irradiance in the plane of the collector (W/m2) </li>
<li>The incidence angle (rad) </li>
<li>The ambient temperature (K) </li>
<li>Fluid input port conditions (i.e. flow rate and temperature)</li>
</ul></p>
<p>The first three inputs should be RealInput Modelica type. </p>
<p><i>Outputs:</i> </p>
<p>Due to the capacities of Modelica language, the value of all the variables used by the solver for the model solution is accessible after simulation. Here just the most interesting ones are mentioned </p>
<p><ul>
<li>Fluid temperatures: inlet, outlet, and the arithmetic mean. </li>
<li>Disaggregated heat flows: input, losses, fluid, and storage. </li>
</ul></p>
<p>------------------------------------------------------------------------------------------------------------------------</p>
</html>"),
    Icon(coordinateSystem(preserveAspectRatio=false, extent={{-140,-120},{140,
            140}}), graphics));
end SimplifiedSolarCollector;
