within DCOL.ThermalStorage.LatentStorages.BaseClasses;
partial model OneDimSemiInf
  "Vessel containing solid and liquid PCM with mass and energy balance"
  import SI = Modelica.SIunits;
  import Modelica.Math.log;
  import Modelica.Constants.pi;
  import Modelica.Constants.inf;
  import Modelica.Constants.small;

  parameter SI.ThermalConductivity k_s=2.16 "Thermal conductivity of the solid water" annotation(Dialog(group="PCM properties"));
  parameter SI.ThermalConductivity k_l=0.556 "Thermal conductivity of the liquid water" annotation(Dialog(group="PCM properties"));
  parameter SI.Density d_s=916.7 "Density of the solid water" annotation(Dialog(group="PCM properties"));
  parameter SI.Density d_l=999.8 "Density of the liquid water" annotation(Dialog(group="PCM properties"));
  parameter SI.SpecificHeatCapacity cp_s=2110 "Specific heat capacity of the solid water" annotation(Dialog(group="PCM properties"));
  parameter SI.SpecificHeatCapacity cp_l=4184 "Specific heat capacity of the liquid water" annotation(Dialog(group="PCM properties"));
  parameter SI.DynamicViscosity mu = 1.79e-3 "Dynamic viscosity of the solid water" annotation(Dialog(group="PCM properties"));
  parameter SI.Temperature Tfus=0.0+273.15 "Fusion temperature of the water" annotation(Dialog(group="PCM properties"));
  parameter SI.Temperature T0=273.15 "Reference temperature" annotation(Dialog(group="PCM properties"));
  parameter SI.SpecificEnthalpy hfus=333.55e3 "Phase change enthalpy" annotation(Dialog(group="PCM properties"));
  parameter SI.CoefficientOfHeatTransfer h_l=568.4 "Convection heat transfer coefficient of liquid water when static" annotation(Dialog(group="PCM properties"));

  parameter SI.Radius r0 = 0.02 "Radius of each coil" annotation(Dialog(group="Geometric properties"));
  parameter SI.Length L = 1 "Length of each colis" annotation(Dialog(group="Geometric properties"));
  //parameter SI.Area A_unit = 1;
  parameter SI.Area A0 = L*pi*r0^2 "External area of ecah coil" annotation(Dialog(group="Geometric properties"));
  parameter SI.Length S_T = 0.1 "Vertical separation between colis" annotation(Dialog(group="Geometric properties"));
  parameter SI.Length S_L = 0.1 "Horizontal separation between coils" annotation(Dialog(group="Geometric properties"));
  parameter Integer n_seg = 10 "Number of segments forming each coil" annotation(Dialog(group="Geometric properties"));
  parameter Integer n_T = 15 "Vertical number of coils" annotation(Dialog(group="Geometric properties"));
  parameter Integer n_L = 15 "Horizontal number of coils" annotation(Dialog(group="Geometric properties"));
  parameter Boolean in_line=true "Configutation of the tubes in the storage (true if In-line; flase if Staggered)" annotation(Dialog(group="Geometric properties"));

  parameter Real eps = 1e-10 annotation(Dialog(group="Adjust value"));

  //Initialization
  parameter SI.Temperature T_l_start = Tfus "Initial liquid water temperature" annotation (Dialog(tab="Initialization")); //0.0 + T0;
  parameter SI.Temperature T_s_start = 0.0 + T0 "Initial solid water temperature" annotation (Dialog(tab="Initialization"));
  parameter SI.MassFlowRate m_flow_start = 5 "Initial mass flow" annotation (Dialog(tab="Initialization"));
  parameter SI.Mass m_l_start = L * (n_L*S_L) * (n_T*S_T) * d_s "Initial mass of liquid water" annotation (Dialog(tab="Initialization"));
  parameter SI.Mass m_s_start = eps "Initial mass of solid water" annotation (Dialog(tab="Initialization"));
  final parameter Integer if_flag_start = 0 "Initial value of the test control1" annotation (Dialog(tab="Initialization"));
  final parameter Integer m_s_flag_start = 0 "Initial value of the test control2" annotation (Dialog(tab="Initialization"));

  final parameter SI.Mass m_l_min=eps "Minimum liquid water mass in the vessel";
  final parameter SI.Area A_i = L * (n_L*S_L) "Transversal area of the water flow";
  final parameter SI.Length S_D = (S_L^2 + (S_T/2)^2)^0.5;
  final parameter Real n = n_T*n_L "Total number of coils";

  SI.Volume V_s;
  SI.Area A;
  SI.Radius r(min=r0);
  SI.Diameter D;
  SI.Diameter D_max;
  SI.Mass m_l(start=m_l_start, stateSelect = StateSelect.prefer);
  SI.Mass m_s(start=m_s_start, stateSelect = StateSelect.always);

  SI.Angle theta;

  SI.InternalEnergy U_s;
  SI.InternalEnergy U_l;
  SI.HeatFlowRate Q_flow_p;
  SI.EnthalpyFlowRate Hb_flow;
  SI.HeatFlowRate Q_conv_tur;
  SI.HeatFlowRate Q_conv_sta;
  SI.HeatFlowRate Q_amb;

  SI.Temperature T_sl(start=Tfus) "Liquid temperature in the vessel";
  SI.Temperature T_l(start=T_l_start, stateSelect = StateSelect.prefer) "Liquid temperature in the vessel";
  SI.Temperature T_s(start=T_s_start, stateSelect = StateSelect.prefer) "Solid temperature in the vessel";
  SI.Temperature T_ext;
  SI.MassFlowRate m_flow_a;
  SI.MassFlowRate m_flow_b;

  SI.NusseltNumber Nu_D;
  SI.ReynoldsNumber Re_D;
  SI.PrandtlNumber Pr;
  SI.CoefficientOfHeatTransfer h_tur;
  SI.CoefficientOfHeatTransfer h_p;
  SI.Velocity v;
  SI.Velocity v_max;

  Real f "Correction factor";
  Real AR "Area ratio";

  Integer if_flag(start=if_flag_start);
  Integer m_s_flag(start=m_s_flag_start);

equation
  assert(D<D_max, "Solid diametre reached its maximum");
  assert(m_l>m_l_min, "No enough liquid");

  U_s = m_s*cp_s*(T_s-T0) - m_s*hfus;
  U_l = m_l*cp_l*(T_l-T0);

  der(U_s) + der(U_l) = Q_flow_p + Hb_flow + Q_amb;

  Q_flow_p = -A*h_p*(T_sl-T_ext);
  Q_conv_tur = A*h_tur*(T_l-T_sl);
  Q_conv_sta = A*h_l*(T_l-T_sl);

  if Q_flow_p<0.0 and m_flow_a<=0.0 then
    if_flag = 1;
    if m_s<=0.0 and T_l>Tfus then
      m_s_flag = 0;
      der(U_l) = Q_flow_p + Q_amb;
    else
      m_s_flag = 1;
      der(U_l) = Hb_flow - cp_l*(T_l-T_sl)*der(m_s) - Q_conv_sta + Q_amb;
    end if;
  elseif not Q_flow_p<0.0 and not m_flow_a<=0.0 then
    if_flag = 2;
    if m_s<=0.0 then
      m_s_flag = 0;
      der(U_l) = Hb_flow + Q_amb;
    else
      m_s_flag = 1;
      der(U_l) = Hb_flow - cp_l*(T_l-T_sl)*der(m_s) - Q_conv_tur + Q_amb;
    end if;
  else
    if_flag = 3;
    if m_s<=0.0 and T_l>Tfus then
      m_s_flag = 0;
      der(U_l) = 0.0 + Q_amb;
    else
      m_s_flag = 1;
      der(U_l) = Hb_flow - cp_l*(T_l-T_sl)*der(m_s) - Q_conv_sta + Q_amb;
    end if;
  end if;

  T_sl = Tfus;
  T_s = (T_sl + T_ext) / 2;

  D_max=min(sqrt(2)*S_T,S_L-0.02);

  if D<=S_T then
    theta=0;
    h_p = (2/log((r+eps)/r0)) * k_s / D;
    f = 0;
    AR = 0;
  else
    theta=acos(S_T/D);
    h_p = (2/log((S_T/2)/r0)) *f* k_s / D;
    f=-1.441*AR+2.455*sqrt(AR)+(r0/(S_T/2))*(3.116*AR-3.158*sqrt(AR));
    AR=1 - ((4/Modelica.Constants.pi)*acos(S_T/D));
  end if;

  m_s = d_s*V_s;
  A = n*L*pi*D - 2*theta*D;
  V_s = n*L*pi*(r^2-r0^2);

  der(m_l) + der(m_s) = m_flow_a + m_flow_b;

  D = 2 * r;
  h_tur = Nu_D * k_l / D;
  Nu_D = 0.33 * Re_D^0.6 * Pr^0.333;

  Re_D = max(d_l*v_max*D / mu, 0.0);

  if in_line==true then
    v_max = S_T * v / (S_T - D);
  else
    v_max = S_T * v / (2*(S_D - D));
  end if;

  Pr = cp_l * mu / k_l;

  v = m_flow_a/(d_l * A_i);

  annotation (Icon(coordinateSystem(preserveAspectRatio=false, extent={{-100,-100},{120,100}}),
                                                                graphics={
        Text(
          extent={{-100,160},{100,100}},
          lineColor={28,108,200},
          textString="%name"),
        Polygon(
          points={{-28,36},{-28,36}},
          lineColor={28,108,200},
          fillColor={255,255,255},
          fillPattern=FillPattern.Solid)}),                      Diagram(
        coordinateSystem(preserveAspectRatio=false, extent={{-100,-100},{100,100}})),
    Documentation(info="<html>
<p>Base class for an ideal vessel with two mediums at liquid-vapor equilibrium with the ability to store mass and energy. The following boundary terms are part of the energy balance and must be specified in an extending class: </p>
<p><b>Qb_flow</b>, e.g., convective or latent heat flow rate across vessel boundary, and</p>
<p><b>Wb_flow</b>, work term.</p>
<p>Further source terms must be defined by an extending class for fluid flow across the segment boundary: </p>
<p><b>Hb_flow</b>, enthalpy flow,</p>
<p><b>mb_flow</b>, mass flow,</p>
<p><b>mbXi_flow_l</b> &AMP; <b>mbXi_flow_v</b>, substance mass flow for liquid and vapor medium respectively.</p>
<p>Since liquid-vapor equilibrium is assumped, temperatures (and pressures) of the vapor medium and the liquid medium are equal, and named <b>T</b> (<b>p</b>).</p>
</html>", revisions="<html>
<p>2016-05-31 <a href=\"mailto:jesus.febres@tekniker.es\">Jes&uacute;s Febres</a> (IK4-TEKNIKER): First implementation</p>
<p><b>Copyright &copy; <a href=\"http://www.tekniker.es/en\">IK4-TEKNIKER</a></b></p>
</html>"),
    experiment(StopTime=6000));
end OneDimSemiInf;
