within DCOL.ThermalStorage.LatentStorages.BaseClasses;
partial model PartialSolidLiquidVessel2
  "Vessel containing solid and liquid PCM with mass and energy balance"
  extends DCOL.ThermalStorage.LatentStorages.Icons.Ice_Water_segments;
  import SI = Modelica.SIunits;
  import Modelica.Media.Interfaces.Choices.IndependentVariables;
  import Modelica.Fluid.Types.Dynamics;
  outer Modelica.Fluid.System system "System wide properties";

  replaceable package Medium_l =
      Modelica.Media.Water.ConstantPropertyLiquidWater
    constrainedby Modelica.Media.Interfaces.PartialMedium
    "Liquid medium in the vessel" annotation (choicesAllMatching=true);

  parameter SI.SpecificEnthalpy hfus=333.55e3;
  parameter SI.SpecificHeatCapacity Cp_l=4184;
  parameter SI.SpecificHeatCapacity Cp_s=2110;
  parameter SI.Density d_s=917.00;
  parameter SI.CoefficientOfHeatTransfer h_l=568.4;
  parameter SI.Diameter d_tube=0.03 annotation(Dialog(group="Geometric properties"));
  parameter SI.Diameter d_crit=0.1 annotation(Dialog(group="Geometric properties"));
  parameter SI.Diameter d_max=sqrt(2)*d_crit annotation(Dialog(group="Geometric properties"));
  parameter SI.Length L_coil=0.066 "Length of each tube segment" annotation(Dialog(group="Geometric properties"));

  // Initialization
  parameter SI.Temperature T_l_start=273.15+20
  "Initial solid temperature"  annotation (Dialog(tab="Initialization"));
  parameter SI.Temperature T_s_start=273.15-0
  "Initial solid temperature" annotation (Dialog(tab="Initialization"));
  parameter SI.AbsolutePressure p_start = system.p_start
  "Initial liquid-vapor equilibrium pressure" annotation(Dialog(tab = "Initialization"));
  parameter SI.MassFraction X_l_start[Medium_l.nX] = Medium_l.X_default
  "Initial X of liquid medium" annotation (Dialog(tab="Initialization", enable=Medium_l.nXi > 0));
  parameter SI.Mass m_l_start=d_s*Modelica.Constants.pi*((d_max/2)^2-(d_tube/2)^2)*L_coil
  "Initial mass of liquid medium" annotation (Dialog(tab="Initialization"));
  parameter SI.Mass m_s_start=0
  "Initial mass of solid medium" annotation (Dialog(tab="Initialization"));
  parameter SI.Diameter d_start=2*((m_s_start / (d_s*3.14*1))+((d_tube/2)^2))^0.5
  "Initial mass of solid medium" annotation (Dialog(tab="Initialization"));

  // Assumptions
  parameter Real eps = 1e-4  annotation(Dialog(tab = "Assumptions", group="aproximation"));
  parameter SI.Temperature T0=273.15;

//   parameter Dynamics energyDynamics=Dynamics.FixedInitial
//   "Formulation of energy balance"  annotation(Evaluate=true, Dialog(tab = "Assumptions", group="Dynamics"));
//   parameter Dynamics massDynamics=Dynamics.FixedInitial
//   "Formulation of mass balance" annotation(Evaluate=true, Dialog(tab = "Assumptions", group="Dynamics"));
//   final parameter Dynamics substanceDynamics=massDynamics
//   "Formulation of mass balance" annotation(Evaluate=true, Dialog(tab = "Assumptions", group="Dynamics"));

  SI.Area A;
  SI.Diameter d(start=d_start);
  //2*((m_s_start / (d_s*3.14*1))+((d_tube/2)^2))^0.5

  SI.Mass m "Mass in the vessel";
  SI.Mass m_l(start=m_l_start, stateSelect = StateSelect.prefer) "Liquid mass of the vessel";
  SI.Mass m_s(start=m_s_start, stateSelect = StateSelect.prefer) "Solid mass in the vessel";
  SI.MassFlowRate mb_flow;

  SI.Mass[Medium_l.nXi] mXi_l;

  SI.InternalEnergy U "Internal energy";
  SI.InternalEnergy U_l "Internal energy";
  SI.InternalEnergy U_s "Internal energy";
  SI.EnthalpyFlowRate Hb_flow "Heat flow across boundaries or energy source/sink";
  SI.HeatFlowRate Qb_flow "Heat flow across boundaries or energy source/sink";

  SI.Energy Q_maxs;
  SI.Time t;
  Real a;

  SI.Temperature T_l(start=T_l_start, stateSelect = StateSelect.prefer) "Liquid temperature in the vessel";
  SI.Temperature T_s(start=T_s_start, stateSelect = StateSelect.prefer) "Solid temperature in the vessel";

  SI.TemperatureDifference T_lN = (T_l-T0)/T0;
  SI.TemperatureDifference T_sN = (T_s-T0)/T0;

  Medium_l.BaseProperties medium_l(
    p(start=p_start),
    T(start=T_l_start),
    Xi(start=X_l_start[1:Medium_l.nXi], stateSelect = StateSelect.prefer))
    "Liquid medium";
    //(start=X_l_start[1:Medium_l.nXi], stateSelect = StateSelect.prefer))

  Boolean flag_l "when there is liquid";
  Boolean flag_l_overHeat "when there is liquid over fusion temperature";
  Boolean flag_s "when there is solid";
  Boolean flag_s_subCold "when there is solid below fusion temperature";

  Integer test;

equation
  // Total quantities
  m = m_l + m_s;
  //m_l = m*x_l;

  // medium_l conditions
  medium_l.T = T_l;
  mXi_l = m_l*medium_l.Xi;

//   U_l = m_l*Cp_l*(T_l-T0);
  U_l = m_l*medium_l.u;
  U_s = m_s*Cp_s*(T_s-T0);
  U = U_l + U_s;

  der(U) - der(m_s)*hfus= Hb_flow + Qb_flow;

 if flag_l and (not flag_s) then
   //Only liquid
   der(U_l) = Hb_flow + Qb_flow;
   //    der(T_s) = 0;//*
   der(m_s) = 0;
   T_s=T0;
//    der(m_l) = 0;
//    d=0;
   t=0;
   a=0;
//    der(d)=0;
   test = 0;
 elseif (not flag_l) and flag_s then
   //Only solid
   //    der(T_l) = 0;//*
   der(U_s) = Hb_flow + Qb_flow;
   der(m_s) = 0;
   der(m_l) = 0;
   t=0;
   a=0;
//    der(d)=0;
   test = 10;
 else
   //Mix of solid and liquid
   if flag_l_overHeat and flag_s_subCold then
     der(U_l) = Hb_flow - A*h_l*(T_l-T_s);
     der(U_s) = Qb_flow + A*h_l*(T_l-T_s);
     //      der(m_s) = 0;//*
     t=0;
     a=0;
     der(d)=0;
     test = 1;
   elseif flag_l_overHeat and (not flag_s_subCold) then
     der(U_l) = Hb_flow - A*h_l*(T_l-T_s);
//      der(U_s) = Qb_flow + A*h_l*(T_l-T_s);//*
     der(T_s) = 0;
      if abs(Qb_flow)>Hb_flow then
        t=(Q_maxs-U_s-U_l)/(-Qb_flow-Hb_flow);
        a=(d-d_tube)/t;
        der(d)=0;
      elseif abs(Qb_flow)<Hb_flow then
        t=(Q_maxs-U_s-U_l)/(-Qb_flow-Hb_flow);
        a=(d-d_tube)/t;
        der(d)=a;
      else
        t=0;
        a=0;
        der(d)=0;
      end if;
     test = 2;
   elseif (not flag_l_overHeat) and flag_s_subCold then
//      der(U_l) = Hb_flow - A*h_l*(T_l-T_s);//*
     der(U_s) = Qb_flow + A*h_l*(T_l-T_s);
     der(T_l) = 0;
      if abs(Qb_flow)>Hb_flow then
        t=(Q_maxs-U_s-U_l)/(-Qb_flow-Hb_flow);
        a=(d-d_tube)/t;
        der(d)=a;
      elseif abs(Qb_flow)<Hb_flow then
        t=(Q_maxs-U_s-U_l)/(-Qb_flow-Hb_flow);
        a=(d-d_tube)/t;
        der(d)=0;
      else
        t=0;
        a=0;
        der(d)=0;
      end if;
     test = 3;
   else
     der(T_s) = 0;
     der(T_l) = 0;
//      - der(m_s)*hfus = Hb_flow + Qb_flow;//*
      if abs(Qb_flow)>Hb_flow then
        t=(Q_maxs-U_s-U_l)/(-Qb_flow-Hb_flow);
        a=(d-d_tube)/t;
        der(d)=a;
      elseif abs(Qb_flow)<Hb_flow then
        t=(Q_maxs-U_s-U_l)/(-Qb_flow-Hb_flow);
        a=(d-d_tube)/t;
        der(d)=a;
      else
        t=0;
        a=0;
        der(d)=0;
      end if;
     test = 4;
   end if;
 end if;

  der(m) = mb_flow;

  m_s=d_s*Modelica.Constants.pi*((d/2)^2-(d_tube/2)^2)*L_coil;
//   Q_maxs=m_max*hfus + A*h_l*(T_l-T_s);
  Q_maxs=m_s*hfus;

  A = Modelica.Constants.pi*d*L_coil;

  annotation (Icon(coordinateSystem(preserveAspectRatio=false), graphics={
        Text(
          extent={{-100,160},{100,100}},
          lineColor={28,108,200},
          textString="%name"),
        Polygon(
          points={{-28,36},{-28,36}},
          lineColor={28,108,200},
          fillColor={255,255,255},
          fillPattern=FillPattern.Solid)}),                      Diagram(
        coordinateSystem(preserveAspectRatio=false)),
    Documentation(info="<html>
<p>Base class for an ideal vessel with two mediums at liquid-vapor equilibrium with the ability to store mass and energy. The following boundary terms are part of the energy balance and must be specified in an extending class: </p>
<p><b>Qb_flow</b>, e.g., convective or latent heat flow rate across vessel boundary, and</p>
<p><b>Wb_flow</b>, work term.</p>
<p>Further source terms must be defined by an extending class for fluid flow across the segment boundary: </p>
<p><b>Hb_flow</b>, enthalpy flow,</p>
<p><b>mb_flow</b>, mass flow,</p>
<p><b>mbXi_flow_l</b> &AMP; <b>mbXi_flow_v</b>, substance mass flow for liquid and vapor medium respectively.</p>
<p>Since liquid-vapor equilibrium is assumped, temperatures (and pressures) of the vapor medium and the liquid medium are equal, and named <b>T</b> (<b>p</b>).</p>
</html>", revisions="<html>
<p>2016-05-31 <a href=\"mailto:jesus.febres@tekniker.es\">Jes&uacute;s Febres</a> (IK4-TEKNIKER): First implementation</p>
<p><b>Copyright &copy; <a href=\"http://www.tekniker.es/en\">IK4-TEKNIKER</a></b></p>
</html>"),
    experiment(StopTime=25));
end PartialSolidLiquidVessel2;
